class ExternalApiController < ApplicationController
  class MissingRequiredParameter < StandardError; end

  before_action :ensure_external_system_param!, except: [:templates]
  before_action :ensure_orders_param!, except: [:templates, :old_set_template]
  before_action :ensure_template_param!, only: [:bulk_update, :old_set_template]

  rescue_from Exception do |exception|
    if Rails.env.production?
      Rails::logger.error "[EXTERNAL API] ERROR - #{exception}"
      render json: { error: 'An unknown error occurred. If this issue persists please contact support.'}, status: :internal_server_error
    else
      raise exception
    end
  end

  def bulk_launch
    order_params = bulk_launch_params

    return invalid_payload_response unless bulk_launch_params_valid?(order_params)

    orders_params = resolve_bulk_launch_ticket_barcodes(order_params)

    order_transaction_data = OrderLaunch.with_order_number_and_ticket_barcode(orders_params[:orders], @source_name)

    transactions_response(order_transaction_data)
  end

  def bulk_set_hot
    order_numbers = order_number_params

    transaction_wrappers = BulkAction::OrderBulkAction::ToggleHotOrderJob.perform_later(
      order_numbers,
      :set_hot,
      source: @source_name,
      transaction_data: {set_hot: true}
    )

    transactions_response(transaction_wrappers)
  end

  def bulk_update
    order_numbers = order_number_params

    transaction_wrappers = BulkAction::OrderBulkAction::ChangeTemplateJob.perform_later(
      order_numbers,
      :set_template,
      source: @source_name,
      transaction_data: {template: @template}
    )

    transactions_response(transaction_wrappers)
  end

  def bulk_release
    order_numbers = order_number_params

    transaction_wrappers = BulkAction::OrderBulkAction::ReleaseOrderJob.perform_later(
      order_numbers,
      :release,
      source: @source_name)

    transactions_response(transaction_wrappers)
  end

  def templates
    render json: TicketTemplate.available_to_external, status: :ok
  end

  concerning :Deprecated do
    #DO NOT REMOVE - STILL USED BY LENSWARE
    def old_set_template
      order_number = old_set_template_params

      transaction_wrappers = BulkAction::OrderBulkAction::ChangeTemplateJob.perform_later(
        [order_number],
        :set_template,
        source: @source_name,
        transaction_data: {template: @template}
      )

      transactions_response(transaction_wrappers)
    end

    def old_set_template_params
      data = params.permit(:order_number)
      data[:order_number]
    end
  end

  private

  def order_number_params
    data = params.permit(orders: [:number])
    data[:orders].map {|order| order[:number]}
  end

  def bulk_launch_params
    data = params.permit(orders: [:launchNumber, :ticketBarcode,  ticketAltIdentifier: {}])

    {
      orders: data[:orders].map do |order_data|
        {
          number: order_data[:launchNumber],
          ticket_barcode: order_data[:ticketBarcode],
          ticket_alt_identifier: order_data[:ticketAltIdentifier]
        }
      end
    }
  end

  def resolve_bulk_launch_ticket_barcodes(order_params)
    alt_identifiers = []

    order_params[:orders].each do |order_params|
      alt_identifiers << order_params[:ticket_alt_identifier][:name] unless order_params[:ticket_alt_identifier].nil?
    end

    if alt_identifiers.length === 0
      return order_params
    end

    alt_ticket_identifiers = TicketStaticDatum.joins(:paperless_ticket)
                                              .where(name: alt_identifiers)
                                              .pluck(:name, :value, 'paperless_tickets.ticket_barcode')
                                              .map {|record| {name: record[0], value: record[1], ticket_barcode: record[2]}}

    order_params[:orders].each do |order_param|
      next unless order_param[:ticket_barcode].nil?
      order_alt = order_param[:ticket_alt_identifier]

      alt_identifier = alt_ticket_identifiers.detect {|identifier| identifier[:name] == order_alt[:name] && identifier[:value] == order_alt[:value]}

      order_param[:ticket_barcode] = if alt_identifier.nil?
                                       order_alt[:value] # return the identifier as is, it will then fail association for E-TICKET_BARCODE_INVALID
                                     else
                                       alt_identifier[:ticket_barcode]
                                     end
    end

    order_params
  end

  def bulk_launch_params_valid?(params)
    order_numbers = params[:orders].map {|order_data| order_data[:number]}
    ticket_identifiers = params[:orders].map do |order_data|
      if order_data[:ticket_barcode]
        order_data[:ticket_barcode]
      else
        if order_data[:ticket_alt_identifier].nil?
          nil
        else
          order_data[:ticket_alt_identifier][:value]
        end
      end
    end

    return false if order_numbers.uniq.length != order_numbers.length
    return false if order_numbers.include?(nil) || ticket_identifiers.include?(nil)
    return false if ticket_identifiers.uniq.length != ticket_identifiers.length

    true
  end

  def ensure_external_system_param!
    @source_name = ExternalSystem.where(token: params.permit(:token)[:token]).pluck(:name)[0]

    if @source_name.nil?
      return render json: { error: "System does not have the right to perform this action." }, status: :forbidden
    end

    true # proceed with the request
  end

  def ensure_template_param!
    data = params.permit(:template)

    return invalid_payload_response if data[:template].nil?

    @template = TicketTemplate.where(template_name: data[:template]).first

    return render json: {error: "No template matches name #{data[:template]}.", code: :INVALID_TEMPLATE}, status: :unprocessable_entity if @template.nil?

    true # proceed with the request
  end

  def ensure_orders_param!
    data = params.permit(orders: [:number])

    if data[:orders].nil?
      return invalid_payload_response
    end

    if data[:orders].length > 100
      return render json: { error: 'Request exceeds the maximum number of simultaneous actions (100 max).', code: :TRANSACTION_LIMIT_EXCEEDED}, status: :unprocessable_entity
    end

    true # proceed with the request
  end

  def invalid_payload_response
    render json: { error: 'One or more parameters are invalid or missing from the payload.', code: :INVALID_PAYLOAD}, status: :unprocessable_entity
  end

  def transactions_response(transaction_wrappers)
    render json: {
      successful: transaction_wrappers[:successful].map do |transaction_data|
        {
          orderNumber: transaction_data[:transaction].paperless_ticket.order_record.number,
          transactionIdentifier: transaction_data[:transaction].transaction_identifier
        }
      end,
      failed: transaction_wrappers[:failed].map do |transaction_data|
        {
          orderNumber: transaction_data[:order_number],
          error: transaction_data[:error],
          code: transaction_data[:code]
        }
      end
    }, status: :ok
  end
end