class OrderRecordsController < ApplicationController
  include TransactionWrapperHelpers

  def self.set_target_before
    super + [:download, :hot_job, :find_my_job, :launch_job, :tags, :change_template, :latest_launch_transaction_id]
  end

  model_data_store(OrderRecord,
                   {create: false, read: true, update: false, delete: true, index: true},
                   {create: false, read: true, update: false, delete: true}
  )

  before_action :ensure_associated_ticket!, only: [:find_my_job, :hot_job, :change_template, :latest_launch_transaction_id]

  def before_filter(filter)
    if filter[:id] == 'paperless_ticket_barcode'
      filter[:id] = 'paperless_ticket.ticket_barcode'
    elsif filter[:id] == 'state'
      value = filter[:value].split(',')
      if value.length === 1
        filter[:value] = value[0]
      else
        filter[:value] = value
      end
    end
    filter
  end

  def before_sort(sort)
    if sort[:id] == 'paperless_ticket_barcode'
      sort[:id] = 'paperless_ticket.ticket_barcode'
    end
    sort
  end

  def before_set_targets
    @models.eager_load(:paperless_ticket)
  end

  def hot_job
    transaction_wrappers = BulkAction::OrderBulkAction::ToggleHotOrderJob.perform_later(
      [@model.number],
      @model.is_hot_job ? :clear_hot : :set_hot,
      transaction_data: {set_hot: !@model.is_hot_job}
    )

    response_for_single_model_bulk_action(transaction_wrappers)
  end

  def find_my_job
    # Find my order is disabled for hot job as it would interfere with the blinking LED for hot job
    return render json: {error: 'Find Order Disabled for Hob Job'}, status: :unprocessable_entity if @model.is_hot_job?
    return render json: {error: 'Must be attached to ticket to invoke find my job'}, status: :unprocessable_entity unless @model.attached_to_ticket?

    transaction_wrappers  = BulkAction::ToggleTicketLedJob.perform_later(
      [@model.paperless_ticket.ticket_barcode],
      :find,
      transaction_data: {
        blink: true,
        blink_params: Setting.instance.default_find_job_blink_settings
      })

    response_for_single_model_bulk_action(transaction_wrappers)
  end

  def launch_job
    result = OrderLaunchQueue.instance.launch_now(@model)

    if result[:success]
      render json: {}, status: :ok
    else
      render json: {error: result[:error], code: result[:code]}, status: :unprocessable_entity
    end
  end

  def tags
    return render json: {error: 'OrderRecord not attached to order'}, status: :unprocessable_entity if @model.rx_data_order_id.nil?

    order_record_data = OrderDataCache.fetch_for_order_records([@model])

    return render json: order_record_data[@model.id], status: :unprocessable_entity if order_record_data[@model.id]["error"]

    render json: order_record_data[@model.id], status: :ok
  end

  def change_template
    ticket_template = TicketTemplate.where(id: params[:template_id]).first
    return render json: {success: false, error: 'Ticket Template does not exist'}, status: :ok if ticket_template.nil?

    transaction_wrappers = BulkAction::OrderBulkAction::ChangeTemplateJob.perform_later(
      [@model.number],
      :set_template,
      transaction_data: {template: ticket_template}
    )

    response_for_single_model_bulk_action(transaction_wrappers)
  end

  def download
    response.headers["Content-Type"] = @model.content_type
    response.headers["Content-Disposition"] = "attachment; filename=\"#{@model.pdf_file_name}\""

    @model.data_file.download do |chunk|
      response.stream.write(chunk)
    end
  end

  def destroy
    unless @model.can_delete?
      return render json: {error:"Order cannot be launched in state #{@model.state}"}, status: :unprocessable_entity
    end

    @model.deassociate_from_ticket_and_destroy

    head :no_content
  end

  def fetch_by_job_number
    params = fetch_params

    return render json: { error: "Invalid Job Number or Ticket Barcode: #{params[:number]}", status: :unprocessable_entity } if params[:number].nil?

    order = OrderRecord.active.find_by(number: params[:number])

    # Get by order first
    @model = order unless order.nil?

    # If order not found then try to find by ticket barcode
    if @model.nil?
      ticket = PaperlessTicket.in_use.where(ticket_barcode: params[:number]).first
      return render json: {error: "Job or Ticket not active or does not exist: #{params[:number]}"} if ticket.nil?
      return render json: {error: "Ticket not attached to an Order: #{params[:number]}"} if ticket.order_record.nil?

      @model = ticket.order_record
    end

    render :show
  end

  def latest_launch_transaction_id
    render json: {transaction_id: @model.paperless_ticket.latest_launch_transaction.id}, status: :ok
  end

  private

  def ensure_associated_ticket!
    return render json: { success: false, error: 'Order not attached to paperless ticket'}, status: :ok unless @model.attached_to_ticket?

    true
  end

  def fetch_params
    params.permit(:number)
  end
end
