class PaperlessTicketsController < ApplicationController
  include NormalizeTicketBarcode
  include TransactionWrapperHelpers

  def self.set_target_before
    super + [:toggle_staged, :release, :current_image, :force_stop_blink]
  end

  model_data_store(PaperlessTicket,
                   {create: false, read: true, update: false, delete: true, index: true},
                   {create: false, read: true, update: false, delete: true})


  def before_set_targets
    filters = params[:filtered]
    return @models if filters.nil?

    type_filter = params[:filtered].find {|p| p[:id] == "state"}
    return @models if type_filter.nil?

    type_filter = type_filter[:value]

    case type_filter
    when 'available'
      @models.available
    when 'staged'
      @models.staged
    when 'in_use'
      @models.eager_load(:order_record).in_use
    else # all
      @models.eager_load(:order_record)
    end
  end

  def before_filter(filter)
    if filter["id"] == "order_number"
      filter["id"] = "order_record.number"
    end

    if filter["id"] == "state"
      filter = nil
    end

    filter
  end

  def toggle_staged
    if @model.staged
      @model.update(staged: false)
    else
      @model.stage
    end

    render json: {}, status: :ok
  end

  def force_stop_blink
    transaction_wrappers = BulkAction::ToggleTicketLedJob.perform_later([@model.ticket_barcode], :stop_find, transaction_data: {blink: false})
    response_for_single_model_bulk_action(transaction_wrappers)
  end

  def release
    return render json: {error: 'ticket not attached to order'}, status: :unprocessable_entity unless @model.attached?
    return render json: {error: 'ticket cannot be released while it is being launched'}, status: :unprocessable_entity if @model.order_record.launching?

    if @model.order_record.failed_launch?
      @model.order_record.deassociate_from_ticket_and_destroy
      render json: {}, status: :ok
    else
      transaction_wrappers = BulkAction::OrderBulkAction::ReleaseOrderJob.perform_later([@model.order_record.number], :release)
      response_for_single_model_bulk_action(transaction_wrappers)
    end
  end

  def destroy
    if @model.attached? || @model.staged
      return render json: {error: 'Ticket is in use'}, status: :conflict
    end

    if @model.destroy
      head :no_content
    else
      render json: @model.errors, status: :conflict
    end
  end

  def current_image
    resp = PaperlessApi::Synchronous::ImageEndpoint.call({}, @model.ticket_barcode)

    return render json: {error: resp[:error]}, status: :unprocessable_entity unless resp[:success]

    response.headers["Content-Type"] = "image/png"
    response.headers["Content-Disposition"] = "inline"
    response.stream.write(resp[:response].body)
  end

  def count
    @available = PaperlessTicket.available.count
    @staged = PaperlessTicket.staged.count
    @in_use = PaperlessTicket.in_use.count
    @total = PaperlessTicket.count

    render :count
  end

  def bulk_add
    tickets = params[:tickets]
    failed_tickets = []

    tickets.map do |ticket|
      barcode = normalize_ticket_barcode(ticket[:ticket_barcode])
      ticket = PaperlessTicket.find_by(ticket_barcode: barcode)

      if ticket.nil?
        PaperlessTicket.create(ticket_barcode: barcode, order_record: nil, staged: false)
      else
        failed_tickets.push({
                              ticket_barcode: barcode,
                              error: 'E-Ticket barcode already in use.'
                            })
      end
    end

    render json: {failed_values: failed_tickets}, status: :ok
  end

  # The following endpoints are accessed via ticket barcode, not ID
  def find_by_barcode
    barcode = normalize_ticket_barcode(params[:ticket_barcode])

    @model = PaperlessTicket.find_by(ticket_barcode: barcode)

    return render json: { not_found: true}, status: :ok if @model.nil?

    render :show
  end

  private

  def model_params
    params.require(:paperless_ticket).permit(:ticket_barcode, :order_record_id)
  end
end
