import * as React from 'react';
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import {PaperlessTicketStore} from "../../stores/PaperlessTicketStore";
import {sendEnqueueSnackbar} from "@rx-suite/ui";
import {MultiScanField} from "../workstations/async_bulk_action_workstation/base/MultiScanField";
import {useBulkListItems} from "../../hooks/useBulkListItems";
import {useClearOnNextScan} from "../../hooks/useClearOnNextScan";
import {useSnackbar} from "notistack";
import {BulkAddRecord, BulkAddTable} from "../workstations/async_bulk_action_workstation/base/BulkAddTable";
import {TempTicketAddInfo} from "../../hooks/table_columns/bulk_add/useBulkAddStatusColumns";
import {AddStatusBadge} from "./AddStatusBadge";

type BulkTicketScanTableProps = {};

export const BulkTicketScanTable: React.FunctionComponent<BulkTicketScanTableProps> = observer(({}) => {
    const {enqueueSnackbar} = useSnackbar();

    const [addList, setAddList, handleFailedAdd] = useBulkListItems([], 'ticket_barcode', true);
    const [onNextScanClear, setOnNextScanClear, clearItems] = useClearOnNextScan();

    const addToAddList = React.useCallback((scanValues: AddRecord) => {
        scanValues['ticket_barcode'] = scanValues['ticket_barcode'].toUpperCase();

        const newAddList = clearItems(addList);

        const duplicateTicketBarcode= addList.find(record => record.ticket_barcode === scanValues['ticket_barcode']);

        if(duplicateTicketBarcode){
            return sendEnqueueSnackbar(t`Ticket Barcode already in list!`, {variant: "error"}, enqueueSnackbar)
        }

        setAddList([...newAddList, {ticket_barcode: scanValues['ticket_barcode'], status: null}])
    }, [addList, onNextScanClear]);

    const bulkAddColumns = React.useMemo(() => {
        return [
            {
                Header: t`E-Ticket Barcode`,
                accessor: 'ticket_barcode',
                id: 'ticket_barcode',
                maxWidth: 200
            },
            {
                Header: t`Status`,
                accessor: (entry: TempTicketAddInfo) => entry,
                id: 'status',
                filterable: false,
                Cell: (cellInfo: { value: TempTicketAddInfo }) => {
                    return (
                        <AddStatusBadge addStatus={cellInfo.value.status} successfulText={t`Successfully Registered`}/>
                    )
                }
            }
        ]
    }, [addList]);

    const handleSubmit = async () => {
        const payload = addList.map((item) => {
            return {
                ticket_barcode: item.ticket_barcode
            }
        })
        try {
            const {_, failed_values} = await PaperlessTicketStore.bulkAddTickets(payload);
            handleFailedAdd(failed_values)
            setOnNextScanClear(true);
        }catch(err){
            sendEnqueueSnackbar(t`Failed to add tickets.`, {variant: "error"}, enqueueSnackbar)
            console.error(err);
        }
    }

    return (
        <React.Fragment>
            <MultiScanField
                fields={[
                    {label: t`E-Ticket Barcode`, key: 'ticket_barcode'}
                ]}
                handleAllScansComplete={(data) => {addToAddList(data as AddRecord)}}
                isLoading={PaperlessTicketStore.isLoading}
            />
            <BulkAddTable
                submitText={t`Register Tickets`}
                handleSubmit={handleSubmit}
                isLoading={PaperlessTicketStore.isLoading}
                data={addList}
                setData={setAddList as React.Dispatch<React.SetStateAction<BulkAddRecord[]>>}
                columns={bulkAddColumns}
            />
        </React.Fragment>
    );
});

type AddRecord = {
    ticket_barcode: string;
    status: any;
}