import * as React from "react";
import {ButtonField, PaddedPaper, ScannerField} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {t} from '@lingui/macro';
import {Grid, makeStyles} from "@material-ui/core";
import {SearchRounded} from "@material-ui/icons";

type OperatorScannerFieldProps = {
    searchFunction: (searchVal: string) => Promise<void>;
    searchFieldPlaceHolderText: string;
    externalLoading?: boolean;
}

const useStyles = makeStyles((theme) => ({
    searchContainer: {
        marginBottom: theme.spacing(1),
    },
    searchBox: {
        width: '90%'
    }
}));

export const OperatorScannerField: React.FunctionComponent<OperatorScannerFieldProps> = observer((props) => {
    const {searchFunction, searchFieldPlaceHolderText, externalLoading} = props;
    const classes = useStyles();

    const [searchFieldValue, setSearchFieldValue] = React.useState<string>("");
    const [isFetching, setIsFetching] = React.useState(false);

    const handleChange = (e: any) => {
        setSearchFieldValue(e.target.value);
    };

    const isLoading = () => {
        return isFetching || externalLoading;
    }

    const onSubmit = async (code: string) => {
        if (isLoading()) {
            return; // wait until fetching is complete before making another request
        }
        const submitValue = code;

        // skip empty search values
        if (submitValue.trim() === '') {
            return;
        }
        setIsFetching(true);
        setSearchFieldValue('');

        await searchFunction(submitValue);

        setIsFetching(false);
    };

    return (
        <PaddedPaper className={classes.searchContainer}>
            <Grid container justify="space-evenly">
                <ScannerField
                    inputConfig={{
                        className: classes.searchBox,
                        label: searchFieldPlaceHolderText,
                        value: searchFieldValue,
                        onChange: handleChange,
                        disabled: isLoading(),
                        autoFocus: true
                    }}
                    scanConfig={{
                        onComplete: onSubmit,
                        onPartial: async (code: string) => {

                            // typing very quickly anywhere on the page could cause an accidental submission

                            // prevent this behaviour by validating that the code in the search field
                            // is equal to the one passed to this method

                            // this proves that the user entered the code in the search field
                            // and sent a request to submit it (using enter key)

                            // this will also stop any invalid scans
                            // (partial scans that are less than 4 characters)

                            if (code !== searchFieldValue) return;

                            await onSubmit(code);
                        },
                        minLength: 4,
                        // add a faster scan detection time to avoid typing
                        // in other fields to be mistaken as a scan
                        averageWaitTime: 20
                    }}
                    global // overwrites onEnterKey to be scanConfig.onComplete or scanConfig.onPartial
                />
                <ButtonField
                    text={t`Search`}
                    size={'large'}
                    buttonType={'submit'}
                    color={'secondary'}
                    startIcon={<SearchRounded/>}
                    onClick={async () => {
                        await onSubmit(searchFieldValue);
                    }}
                    loading={isLoading()}
                    disabled={searchFieldValue==""}
                />
            </Grid>
        </PaddedPaper>
    );
});