import * as React from 'react';
import {observer} from "mobx-react-lite";
import {Box, Dialog, DialogContent, DialogTitle, Grid, makeStyles, Typography} from "@material-ui/core";
import {t} from "@lingui/macro";
import {ButtonField, Conditional, SelectField, useModelContext} from "@rx-suite/ui";
import {TicketTemplateStore} from "../../../stores/TicketTemplateStore";
import {OrderRecordModel} from "../../../stores/OrderRecordStore";
import {useSnackbar} from "notistack";
import {InfoRounded} from "@material-ui/icons";

type ChangeTicketTemplateDialogProps = {
    open: boolean,
    handleClose: () => void;
}

const useStyles = makeStyles((theme) => ({
    icon: {
        fontSize: '1.25rem'
    },
    templateName: {
        fontWeight: 'bold'
    }
}));

export const ChangeTicketTemplateDialog: React.FunctionComponent<ChangeTicketTemplateDialogProps> = observer(({open, handleClose}) => {
    const model = useModelContext() as OrderRecordModel;
    const {enqueueSnackbar} = useSnackbar();

    const classes = useStyles();

    const [selectedTemplateId, setSelectedTemplateId] = React.useState<number|null>(null);
    const [templateUpdating, setTemplateUpdating] = React.useState<boolean>(false);

    const templateOptions = [
        ...TicketTemplateStore.getAll()
            .filter(template => !template.isRelease)
            .map(template => {
                return { label: template.process_name, value: template.id }
            })
    ];

    const currentTemplate = React.useMemo(() => {
        if(TicketTemplateStore.isLoading) {
            return t`loading...`
        }

        if(model.ticket_template_id === null){
            return t`No Template`
        }

        return templateOptions.find(option => option.value === model.ticket_template_id)?.label;

    }, [TicketTemplateStore.models, model.ticket_template_id]);


    const handleConfirmTemplateChange = React.useCallback(async(selectedTemplateId: number) => {
        if(selectedTemplateId === null){
            return;
        }

        setTemplateUpdating(true);
        try {
            await model.changeTemplate(selectedTemplateId);
            enqueueSnackbar(t`Template change requested.`, { variant: 'success' });
            handleClose();
        } catch (err) {
            console.error(err)
            enqueueSnackbar(t`Something went wrong while requesting template change.`, { variant: 'error' });
        }

        setTemplateUpdating(false);
    }, [model.ticket_template_id]);


    return (
        <Dialog open={open} onClose={handleClose} aria-labelledby="form-dialog-title" fullWidth={true} maxWidth={'xs'}>
            <DialogTitle id="form-dialog-title">Change Ticket Template</DialogTitle>
            <DialogContent>
                <Conditional condition={!TicketTemplateStore.isLoading}>
                    <Box display={'flex'} alignItems={'center'} mb={0.5}>
                        <InfoRounded className={classes.icon}/>
                        <Box ml={0.5}>
                            <Typography>{t`Currently Displayed:`}</Typography>
                        </Box>
                        <Box ml={0.5}>
                            <Typography className={classes.templateName}>{currentTemplate}</Typography>
                        </Box>
                    </Box>
                </Conditional>
                <Box>
                    <SelectField
                        label={t`Set Template...`}
                        onChange={(id: any, value: any) => {
                            setSelectedTemplateId(value)
                        }}
                        value={selectedTemplateId}
                        options={templateOptions}
                        loading={TicketTemplateStore.isLoading}
                        disabled={templateUpdating}
                    />
                </Box>
                <Box mt={2}>
                    <Grid container justifyContent={"center"}>
                        <ButtonField
                            text={t`Update`}
                            variant={'contained'}
                            color={'secondary'}
                            onClick={()=>{ handleConfirmTemplateChange(selectedTemplateId as number)}}
                            size={'large'}
                            disabled={TicketTemplateStore.isLoading || selectedTemplateId === null}
                            loading={templateUpdating}
                        />
                        <ButtonField
                            text={t`Cancel`}
                            variant={'contained'}
                            onClick={handleClose}
                            size={'large'}
                        />
                    </Grid>
                </Box>
            </DialogContent>
        </Dialog>
    )
});
