import * as React from "react";
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import {Add, InfoRounded} from "@material-ui/icons";
import Box from "@material-ui/core/Box";
import Typography from "@material-ui/core/Typography";
import {Grid, makeStyles} from "@material-ui/core";
import {grey} from "@material-ui/core/colors";
import {ButtonField, ScannerField} from "@rx-suite/ui";
import {useMemo, useState} from "react";
import {Schema} from "yup";

type ScanFieldProp = {
    label: string;
    key: string;
    validation?: Schema;
}

type MultiScanFieldProps = {
    fields: ScanFieldProp[];
    handleAllScansComplete: (columnValues: Record<string, string>) => void;
    instructions?: string;
    isLoading: boolean;
}

const useStyles = makeStyles((theme) => ({
    scanFieldWrapper: {
      flexGrow: 1
    },
    buttonWrapper: {
        width: 'fit-content'
    },
    infoSection: {
        color: grey[600],
        fontSize: '0.9rem'
    }
}));

export const MultiScanField: React.FunctionComponent<MultiScanFieldProps> = observer(({
                                                                                          fields,
                                                                                          handleAllScansComplete,
                                                                                          isLoading,
                                                                                          instructions
}) => {
    const classes = useStyles();
    const scannerFieldRefs: { [key: string]: HTMLInputElement } = {};

    const defaultScanFieldValues = useMemo(() => {
        return fields.reduce((hash: Record<string, string>, field) => {
            hash[field.key] = '';
            return hash;
        }, {})
    }, [fields])

    // The state of this doesnt matter, as long as it changes. Whenever it changes we trigger calculations on what input to focus on next.
    const [completeScan, setCompleteScan] = useState(false);
    const [scanFieldValues, setScanFieldValues] = React.useState<Record<string, string>>(defaultScanFieldValues);

    const allFieldsComplete = useMemo(() => {
        return Object.values(scanFieldValues).every((scanFieldValue) => scanFieldValue !== '')
    }, [scanFieldValues])

    React.useEffect(() => {
        const scanField = fields.find((field) => {
            return scanFieldValues[field.key] === '';
        })

        if(!scanField) { return }

        const scannerFieldRef = scannerFieldRefs[scanField?.key];
        if(scannerFieldRef){
            scannerFieldRef.focus();
        }
    }, [completeScan])

    const handleComplete = () => {
        setCompleteScan(prevState => !prevState);

        if(!allFieldsComplete) { return }

        handleAllScansComplete(scanFieldValues)
        setScanFieldValues({...defaultScanFieldValues});
    }

    const attachScannerFieldRefs = (key: string, element: HTMLInputElement) => {
        scannerFieldRefs[key] = element;
    }

    return (
        <Grid direction={'row'} container>
            <Grid direction={'column'} className={classes.scanFieldWrapper}>
                {fields.map((field) => {
                    return <ScannerField
                        inputConfig={{
                            label: field.label,
                            value: scanFieldValues[field.key],
                            onChange: (e: any) => {
                                //https://react.dev/learn/updating-objects-in-state
                                const tempHash = {...scanFieldValues};
                                tempHash[field.key] = e.target.value;
                                setScanFieldValues(tempHash);
                            },
                            disabled: isLoading,
                            autoFocus: false
                        }}
                        scanConfig={{
                            onComplete: handleComplete,
                            minLength: 4,
                            averageWaitTime: 20
                        }}
                        global
                        inputRefOverride={(elm: any) => attachScannerFieldRefs(field.key, elm)}
                    />
                })}
                {instructions &&
                    <Box display={'flex'} alignItems={'center'} mb={2}>
                        <InfoRounded className={classes.infoSection}/>
                        <Box ml={0.5}>
                            <Typography className={classes.infoSection}>{instructions}</Typography>
                        </Box>
                    </Box>
                }
            </Grid>
            <Grid container alignItems={'flex-end'} className={classes.buttonWrapper}>
                <ButtonField
                    text={t`Add`}
                    variant={'contained'}
                    color={'secondary'}
                    startIcon={<Add/>}
                    disabled={!allFieldsComplete}
                    onClick={handleComplete}
                    loading={isLoading}
                />
            </Grid>
        </Grid>
    );
});

