import {observer, useLocalObservable} from "mobx-react-lite";
import {OrderFindQueryModel} from "../../../stores/OrderFindQueryStore";
import React from "react";
import {
    AntdConfig,
    Builder,
    BuilderProps,
    Config, FieldOrGroup, Fields,
    ImmutableTree,
    Query,
    Utils as QbUtils
} from "@react-awesome-query-builder/antd";
import "@react-awesome-query-builder/antd/css/styles.css";
import {computedFn} from "mobx-utils";
import equal from 'deep-equal'
import {TagModel, TagStore} from "../../../stores/TagStore";
import {TicketTemplateModel, TicketTemplateStore} from "../../../stores/TicketTemplateStore";

type QueryBuilderProps = {
    model: OrderFindQueryModel
}

const extractFieldDef = computedFn(function getDef(tagModel: TagModel): FieldOrGroup {
    let type;
    switch(tagModel.value_type) {
        case "barcode":
        case "string":
            type = 'text'
            break;
        case "number":
            type = 'number'
            break;
        case "boolean":
            type = 'boolean'
            break;
    }
    return {
        type: type,
        label: tagModel.tag
    }
})

const getConfig = computedFn(function getConfig(tagFields: TagModel[], templateFields: TicketTemplateModel[]): Config {
    return ({
        ...AntdConfig,
        fields: {
            template: {
                type: 'select',
                label: 'template',
                fieldSettings: {
                    listValues: templateFields.map((template) => {
                        return {
                            value: template.template_name,
                            label: template.template_name
                        }
                    })
                }
            },
            tags: {
                type: '!struct',
                label: 'Tags',
                subfields: tagFields.reduce((prevValue: Fields, tag) => {
                    prevValue[tag.tag] = extractFieldDef(tag);
                    return prevValue;
                }, {})
            }
        }
    } as Config)
});

export const QueryBuilder: React.FunctionComponent<QueryBuilderProps> = observer(({model}) => {
    const config = getConfig(TagStore.models, TicketTemplateStore.models);

    const renderBuilder = (props: BuilderProps) => (
        <div className="query-builder-container">
            <div className="query-builder">
                <Builder {...props} />
            </div>
        </div>
    );

    const treeState = useLocalObservable(() => {
        return {
            workingTree: (model.viewModel.query && Object.keys(model.viewModel.query?.logic || {}).length > 0) ?
                QbUtils.loadFromJsonLogic(model.viewModel.query.logic, config) as ImmutableTree :
                QbUtils.loadFromJsonLogic({
                    and: []
                }, config) as ImmutableTree,
            updateWorkingTree(newTree: ImmutableTree) {
                const serialized = QbUtils.jsonLogicFormat(newTree, config);
                if (serialized && (!serialized.errors || (serialized.errors.length === 0)) && !equal(model.viewModel.query?.logic, serialized.logic, {strict: true})) {
                    model.viewModel.query = serialized;
                }
            }
        }
    }, [model, config])

    return <React.Fragment>
        <Query
            value={treeState.workingTree}
            onChange={treeState.updateWorkingTree}
            renderBuilder={renderBuilder}
            {...config}
        />
    </React.Fragment>
})