import * as React from "react";
import {Box, Typography} from "@material-ui/core";
import {BreadModel, PaddedPaper, SelectField} from "@rx-suite/ui";
import {OrderRecordModel, OrderRecordStore} from "../../../stores/OrderRecordStore";
import {OrderRecordForm} from "../form/OrderRecordForm";
import {t} from "@lingui/macro";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import {LaunchNowButton} from "../../../components/order_records/LaunchNowButton";
import {useCallback} from "react";
import {Model} from "@rx-suite/data";
import {Column, Filter, ReactTableFunction} from "react-table";
import {LatestLaunchTransactionButton} from "../../../components/order_records/LatestLaunchTransactionButton";
import {OrderStateBadge} from "../../../components/order_records/OrderStateBadge";
import {DeleteOrderButton} from "../../../components/order_records/DeleteOrderButton";

export const LaunchingOrderRecordsBreadModel: React.FunctionComponent = () => {
    const columns: Column[] = React.useMemo(() => {
        return [
            {
                Header: t`Launch Number`,
                accessor: (orderRecord: OrderRecordModel) => orderRecord,
                id: 'launch_number',
                sortable: false,
                filterable: true,
                Cell: (cellInfo: {value: OrderRecordModel }) => {
                    const orderRecordModel = cellInfo.value;
                    const number = orderRecordModel.launch_number;

                    return <Typography variant={'body1'}>{number}</Typography>;
                },
                maxWidth: 250
            },
            {
                Header: t`E-Ticket Barcode`,
                accessor: (orderRecord: OrderRecordModel) => orderRecord,
                id: 'ticket_barcode',
                sortable: false,
                filterable: false,
                Cell: (cellInfo: {value: OrderRecordModel }) => {
                    const orderRecordModel = cellInfo.value;
                    const barcode = orderRecordModel.paperless_ticket_barcode;

                    return <Typography variant={'body1'}>{barcode}</Typography>;
                },
                maxWidth: 250
            },
            {
                Header: t`Entry Time`,
                accessor: (orderRecord: OrderRecordModel) => orderRecord,
                id: 'entry_time',
                sortable: false,
                Cell: (cellInfo: {value: OrderRecordModel }) => {
                    const orderRecordModel = cellInfo.value;
                    const entry_time = orderRecordModel.entry_time;
                    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;

                    return <Typography variant={'body1'}>{systemSettingModel.printDateTime(entry_time)}</Typography>;
                },
                maxWidth: 200
            },
            {
                Header: t`State`,
                accessor: (orderRecord: OrderRecordModel) => orderRecord,
                id: 'state',
                sortable: true,
                Cell: (cellInfo: {value: OrderRecordModel }) => {
                    return <OrderStateBadge model={cellInfo.value}/>
                },
                Filter: ({ filter, onChange }: { filter: Filter, onChange: ReactTableFunction }) => (
                    <SelectField
                        options={[
                            {label: t`Waiting For Launch`, value: 'waiting_for_launch'},
                            {label: t`Failed To Launch`, value: 'failed_launch'},
                            {label: t`Launching`, value: 'launching'}
                        ]}
                        onChange={(name, value: string) => {
                            onChange(value);
                        }}
                        value={filter ? filter.value : ""}
                        multiple
                    />),
                filterable: true
            }
        ];
    }, []);

    const customActions = useCallback((model: Model) => {
        return (
            <Box display={"flex"} justifyContent={"space-evenly"}>
                <LaunchNowButton model={model as OrderRecordModel}/>
                <LatestLaunchTransactionButton model={model as OrderRecordModel}/>
                <DeleteOrderButton model={model as OrderRecordModel}/>
            </Box>
        )
    }, []);

    return (
        <React.Fragment>
            <PaddedPaper>
                <BreadModel
                    store={OrderRecordStore}
                    Form={OrderRecordForm}
                    columns={columns}
                    mountCreate={false}
                    mountEdit={false}
                    mountRead={false}
                    tableProps={
                        {
                            suppressWrapper: true,
                            showCreateButton: false,
                            defaultSorted:[{id: "entry_time", desc: true}],
                            defaultFiltered: [{id: 'state', value: ['launching', 'waiting_for_launch', 'failed_launch'], strictMatchFilter: true}],
                            noLinkColumns: ['launch_number', 'ticket_barcode', 'entry_time', 'state'],
                            replacementActions: {
                                Read: () => null,
                                Edit: () => null,
                                Delete: () => null
                            },
                            customActions: customActions,
                            rememberFetchParams: false,
                            actionColumnWidth: 400
                        }
                    }
                />
            </PaddedPaper>
        </React.Fragment>
    )
};