import * as React from 'react';
import {observer} from "mobx-react-lite";
import {useModelContext} from "@rx-suite/ui";
import {OrderRecordModel, Side} from "../../../../stores/OrderRecordStore";
import {t} from "@lingui/macro";
import ReactTable from "react-table";
import {Box, LinearProgress} from "@material-ui/core";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import {InlineWarning} from "../../../../components/generic/InlineWarning";


type FormattedOrderData = ReadonlyArray<{tag: string, left: string | null, right: string | null}>;
type ErrorOrderData = { error: string };

export const OrderRecordData: React.FunctionComponent = observer(() => {
    const model = useModelContext() as OrderRecordModel;

    const [orderRecordTags, setOrderRecordTags] = React.useState<FormattedOrderData|ErrorOrderData|null>(null);

    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;
    const displayRightLensFirst = systemSettingModel?.displayRightLensFirst();

    const leftColumn = {
        Header: t`Left`,
        accessor: 'left'
    };

    const rightColumn = {
        Header: t`Right`,
        accessor: 'right'
    };

    const columns = React.useMemo(() => {
        return [{
                Header: t`Tag`,
                accessor: 'tag',
                filterable: true
            },
            displayRightLensFirst ? rightColumn : leftColumn,
            displayRightLensFirst ? leftColumn : rightColumn
        ];
    }, []);

    const formatTags = React.useCallback((tags): FormattedOrderData => {

        const displayObj: {
            [tagName: string]: {
                left: string | null;
                right: string | null;
            }
        } = {};

        for (const side of [Side.LEFT, Side.RIGHT]) {
            const jobDataObj = tags[side];
            if (!jobDataObj) continue; // no information for this job side
            // iterate over each tag name/value of this job side
            for (const dataKey in jobDataObj) {
                if (!dataKey || !dataKey.length) continue; // no tag name associated to value
                const value = jobDataObj[dataKey];
                if (!(dataKey in displayObj)) {
                    // entry for tag name does not exist, create it
                    displayObj[dataKey] = {
                        [Side.LEFT]: '',
                        [Side.RIGHT]: ''
                    };
                }
                if (!value) continue; // if the value is null, keep the default string
                // set the value associated to this tag name for this job side
                displayObj[dataKey][side] = value.toString();
            }
        }

        return Object.keys(displayObj).map(tagKey => {
            const displayObjValues = displayObj[tagKey];
            return {
                tag: tagKey,
                [Side.LEFT]: displayObjValues.left,
                [Side.RIGHT]: displayObjValues.right
            };
        });

    }, []);

    React.useEffect(() => {
        const fetchTags = async () => {
            try {
                const tags = await model.fetchTags();
                const formattedTags = formatTags(tags);
                setOrderRecordTags(formattedTags);
            } catch (e) {
                setOrderRecordTags({ error: e });
            }
        };

        fetchTags();
    }, []);

    if (!orderRecordTags) {
        return <LinearProgress/>;
    }

    if ('error' in orderRecordTags) {
        return (
            <Box mt={2} mb={2}>
                <InlineWarning text={t`Something went wrong while retrieving this order's data.`}/>
            </Box>
        )
    }

    return (
        <ReactTable
            data={orderRecordTags}
            columns={columns}
            sortable={false}
            showPagination={false}
            defaultPageSize={orderRecordTags.length}
            minRows={orderRecordTags.length <= 10 ? 10 : orderRecordTags.length} // if empty, pad the table with 10 rows
        />
    );
});
