import * as React from 'react';
import {observer} from "mobx-react-lite";
import {Conditional, FormProps, ModelActionButton, ModelProvider, PaddedPaper} from "@rx-suite/ui";
import {PaperlessApiTransactionModel, PaperlessApiTransactionStore} from "../../stores/PaperlessApiTransactionStore";
import {
    Box,
    Divider,
    Paper,
    Typography,
    Accordion,
    AccordionSummary,
    AccordionDetails,
    makeStyles, alpha
} from "@material-ui/core";
import {t} from "@lingui/macro";
import {ExpandMore, Refresh, ReportProblem} from "@material-ui/icons";
import ReactJson from 'react-json-view'
import {grey} from "@material-ui/core/colors";
import {useEffect} from "react";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";

type PaperlessApiTransactionFormProps = {
    model: PaperlessApiTransactionModel;
} & Omit<FormProps, 'model'>

const useStyles = makeStyles((theme) => ({
    background: {
        backgroundColor: alpha(grey[400], 0.05)
    },
    danger: {
        backgroundColor: theme.palette.error.main
    }
}));

export const PaperlessApiTransactionForm: React.FunctionComponent<PaperlessApiTransactionFormProps> = observer(({model}) => {
    const classes = useStyles();

    useEffect(() => {
        PaperlessApiTransactionStore.subscribe([model.getId()])
        return () => {
            PaperlessApiTransactionStore.unsubscribe();
        }
    }, []);

    const fields = React.useMemo(() => {
        return [
            {
                label: t`Type`,
                value: model.resolved_transaction_type
            },
            {
                label: t`State`,
                value: model.state
            },
            {
                label: t`Source`,
                value: model.source
            },
            {
                label: t`Created at`,
                value: (SystemSettingStore.selectedModel as SystemSettingModel).printDateTime(model.created_at)
            },
            {
                label: t`Completed at`,
                value: model.isProcessed ? (SystemSettingStore.selectedModel as SystemSettingModel).printDateTime(model.finished_at)  : t`ongoing`
            },
            {
                label: t`E-Ticket Command Duration`,
                value: model.isProcessed ? `${model.duration}s` : t`ongoing`
            },
        ]
    }, [model, SystemSettingStore.selectedModel]);

    return (
        <ModelProvider value={model}>
            <PaddedPaper>
                <Box mb={3} display={'flex'} justifyContent={'space-between'}>
                    <Paper variant={'outlined'}>
                        <Box className={classes.background} p={2}>
                            <Typography variant={'h5'}>{t`UUID:` + ' ' + model.transaction_identifier}</Typography>
                            <Typography variant={'h6'}>{t`Associated to ticket:` + ' ' + model.associated_ticket_barcode}</Typography>
                        </Box>
                    </Paper>
                    <Box>
                        <Conditional condition={!model.isProcessed}>
                            <ModelActionButton
                                name={'cancelTransaction'}
                                text={t`Cancel Transaction`}
                                variant={'contained'}
                                size={'large'}
                                className={classes.danger}
                                startIcon={<ReportProblem/>}
                            />
                        </Conditional>
                        <ModelActionButton
                            name={'refreshStatus'}
                            text={t`Refresh`}
                            variant={'contained'}
                            color={'primary'}
                            size={'large'}
                            startIcon={<Refresh/>}
                        />
                    </Box>
                </Box>
                <Divider/>
                <Box p={2} display={'flex'} flexDirection={'column'} justifyContent={'flex-start'}>
                    <Box display={'flex'} flexDirection={'row'} justifyContent={'flex-start'}>
                        {
                            fields.map((field, i) => {
                                return (
                                    <>
                                        <Box mr={3} ml={i===0 ? 0 : 3}>
                                            <Typography variant={'h6'}>{field.label}</Typography>
                                            <Typography variant={'h5'}>{field.value}</Typography>
                                        </Box>
                                        {(i !== fields.length-1) && <Divider orientation="vertical" flexItem />}
                                    </>
                                )
                            })
                        }
                    </Box>
                    <Box mt={3} display={'flex'} flexDirection={'row'} justifyContent={'flex-start'}>
                        <Box width={'40%'}>
                            <Accordion>
                                <AccordionSummary expandIcon={<ExpandMore />}>
                                    <Typography>{t`Request Body`}</Typography>
                                </AccordionSummary>
                                <AccordionDetails>
                                    {!model.request_body
                                        ? 'null'
                                        : <ReactJson
                                            src={JSON.parse(model.request_body)}
                                            name={null}
                                            displayObjectSize={false}
                                            displayDataTypes={false}
                                        />
                                    }
                                </AccordionDetails>
                            </Accordion>
                        </Box>
                        <Box ml={3} width={'40%'}>
                            <Accordion >
                                <AccordionSummary expandIcon={<ExpandMore />}>
                                    <Typography>{t`Response Body`}</Typography>
                                </AccordionSummary>
                                <AccordionDetails>
                                    {!model.response_body
                                        ? 'null'
                                        : <ReactJson
                                            src={JSON.parse(model.response_body)}
                                            name={null}
                                            displayObjectSize={false}
                                            displayDataTypes={false}
                                        />
                                    }
                                </AccordionDetails>
                            </Accordion>
                        </Box>
                    </Box>
                </Box>
            </PaddedPaper>
        </ModelProvider>
    );
});
