import * as React from 'react';
import {observer} from "mobx-react-lite";
import {Box, Divider, Typography} from "@material-ui/core";
import {t, Trans} from "@lingui/macro";
import {Conditional, HintButton, ModelInput, ModelSelect, useModelContext} from "@rx-suite/ui";
import {AutoLaunchSwitch} from "../../../components/settings/AutoLaunchSwitch";
import {SettingModel} from "../../../stores/SettingStore";
import Alert from "@material-ui/lab/Alert";
import {HintList} from "../../../components/generic/HintList";

export const OrderLaunchingTab: React.FunctionComponent = observer(() => {
    const model = useModelContext() as SettingModel;

    const setAutoLaunch = React.useCallback((checked: boolean) => {
        model.nestedAssignment('auto_launch', checked);
    }, [model]);

    const orderPairingModeHint = [
        {title: t`Manual Pairing`, text: t`Orders will be created by user input and will be assigned to a specific e-ticket.`},
        {title: t`Automatic Pairing`, text: t`Orders will be automatically created from PDFs and will be assigned to any staged ticket.`}
    ]

    return (
        <React.Fragment>
            <Box>
                <Box mt={1}>
                    <Typography variant={'h6'}>
                        <Trans>Order Ticket Pairing</Trans>
                    </Typography>
                </Box>
                <ModelSelect name={'order_pairing_mode'}
                             options={[
                                 {label: t`Automatic`, value: 'automatic'},
                                 {label: t`Manual`, value: 'manual'}
                             ]}
                             hint={
                                 {
                                     title: t`Order Ticket Pairing Mode`,
                                     children: <HintList items={orderPairingModeHint}/>
                                 }
                             }
                />
            </Box>
            <Conditional condition={!model.viewModel.isManualPairingMode}>
                <Box mt={2}>
                    <Box mt={1}>
                        <Typography variant={'h6'}>
                            <Trans>Launch Queue</Trans>
                        </Typography>
                    </Box>
                    <Conditional condition={!model.allowEditAutoLaunch}>
                        <Box mb={2} mt={1}>
                            <Alert severity="info">
                                <Typography>{t`Auto Launch cannot be enabled while some settings are missing.`}</Typography>
                            </Alert>
                        </Box>
                    </Conditional>
                    <Box mb={3} width={'100%'} display={'flex'} flexDirection={'row'} justifyContent={'start'} alignItems={'center'}>
                        <Box flex={1}>
                            <AutoLaunchSwitch
                                checked={model.viewModel.dig('auto_launch')}
                                setChecked={setAutoLaunch}
                                disabled={!model.allowEditAutoLaunch}
                            />
                        </Box>
                        <Box>
                            <HintButton
                                title={t`Auto Launch`}
                                text={t`When enabled, jobs will automatically be paired to available e-tickets.`}
                            />
                        </Box>
                        <Box flex={3}></Box>
                    </Box>
                    <ModelInput
                        label={t`Path to Order Folder`}
                        name={'job_folder_path'}
                        hint={
                            {
                                title: t`Path to Order Folder`,
                                text: t`Path to the folder where the application will look for order PDFs`
                            }
                        }
                    />
                    <ModelInput
                        label={t`Order PDF regex`}
                        name={'job_regex'}
                        hint={
                            {
                                title: t`Order PDF regex`,
                                text: t`Regex used to extract the order number from the file name`
                            }
                        }
                    />
                    <Box mt={2}>
                        <ModelInput
                            label={t`Maximum Launch (per Minute)`}
                            name={'max_launch_records'}
                            hint={
                                {
                                    title: t`Maximum Launch (per Minute)`,
                                    text: t`The maximum amount of orders that can be launched during a one minute window. Setting this too high can have negative performance impacts.`
                                }
                            }
                        />
                    </Box>
                </Box>
            </Conditional>
        </React.Fragment>
    );
});
