import * as React from "react";
import {observer} from "mobx-react-lite";
import {TicketTemplateModel} from "../../../stores/TicketTemplateStore";
import {makeColumn, AssociatedModelTable, BreadModelListBadge, SelectField} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {TicketTemplateFieldModel, TicketTemplateFieldStore} from "../../../stores/TicketTemplateFieldStore";
import {TagStore} from "../../../stores/TagStore";
import {TicketTemplateFieldForm} from "./TicketTemplateFieldForm";
import {Box, Divider} from "@material-ui/core";
import Typography from "@material-ui/core/Typography";
import {Filter, ReactTableFunction} from "react-table";

type TicketTemplateLayoutProps = {
    model: TicketTemplateModel;
}

export const TicketTemplateFields: React.FunctionComponent<TicketTemplateLayoutProps> = observer(({model}) => {
    TagStore.getAll();

    const columns = React.useMemo(() => {
        return [
            makeColumn('zone_identifier', t`Zone ID`, "string"),
            {
                Header: t`Field Type`,
                accessor: (field: TicketTemplateFieldModel) => field,
                id: 'field_type',
                Cell: (cellInfo: { value: TicketTemplateFieldModel }) => {
                    return <BreadModelListBadge text={cellInfo.value.resolvedFieldType}/>;
                },
                Filter: ({ filter, onChange }: { filter: Filter, onChange: ReactTableFunction }) => (
                    <SelectField
                        options={[
                            {label: t`Job Data`, value: 'job_data'},
                            {label: t`Ticket Static Data`, value: 'ticket_data'},
                        ]}
                        onChange={(name, value: string) => {
                            onChange(value);
                        }}
                        value={filter ? filter.value : ""}
                        clearable
                    />),
                filterable: true,
                maxWidth: 200
            },
            {
                Header: t`Field Value`,
                accessor: (field: TicketTemplateFieldModel) => field,
                id: 'field_value',
                sortable: false,
                filterable: false,
                Cell: (cellInfo: { value: TicketTemplateFieldModel }) => {
                    const field = cellInfo.value;

                    if(field.isTicketDataField){
                        return field.static_field;
                    }

                    if(field.isJobDataField){
                        const tagModel = TagStore.models.find(t => Number(t.getId()) == cellInfo.value.tag_id);

                        if(!tagModel) {
                            return null;
                        }

                        return `${tagModel.tag} (${field.side})`;
                    }

                    return null;
                }
            },
            {
                Header: t`Formatter (Regex)`,
                accessor: 'formatter_regex',
                id: 'regex',
                filterable: false,
                sortable: false
            }
        ]
    }, [TagStore.models]);


    return (
        <Box mt={1}>
            <Divider/>
            <Box mb={1}>
                <Typography variant={'h6'}>{t`Template Fields`}</Typography>
            </Box>
            <AssociatedModelTable
                baseModel={model}
                columns={columns}
                store={TicketTemplateFieldStore}
                associatedColumnName={'new_template_fields'}
                DialogForm={TicketTemplateFieldForm}
                dialogTitle={t`Add Template Field`}
                createButtonText={t`Add Template Field`}
                fixedHeight={false}
                defaultFiltered={[{id: "ticket_template_id", value: model.id}]}
                noLinkColumns={['zone_identifier', 'field_type', 'field_value', 'regex']}
                replacementActions={
                    {
                        Read: () => null
                    }
                }
                suppressWrapper
            />
        </Box>
    )
})