import * as React from "react";
import {Observer, observer} from "mobx-react-lite";
import {TicketWorkstationModel} from "../../stores/TicketWorkstationStore";
import {
    makeColumn,
    ModelDialog,
    ModelDialogTable,
    useFlag,
    ModelContext, FilterParams, SelectField, ModelProvider, PaddedPaper, WorkstationHelper
} from "@rx-suite/ui";
import {OrderFindQueryModel, OrderFindQueryStore} from "../../stores/OrderFindQueryStore";
import {useMemo, useState} from "react";
import {t} from "@lingui/macro";
import {DataStore, Model} from "@rx-suite/data";
import {WorkstationHeader} from "../../components/workstations/WorkstationHeader";
import {faPlus, faSearch} from "@rx-suite/pro-solid-svg-icons";
import {FontAwesomeIcon} from "@fortawesome/react-fontawesome";
import {QueryActions} from "../../components/workstations/order_find_workstation/QueryActions";
import {QueryStatusBadge} from "../../components/workstations/order_find_workstation/QueryStatusBadge";
import {Grid, makeStyles} from "@material-ui/core";
import Typography from "@material-ui/core/Typography";
import {
    OrderFindQueryDialog
} from "../../components/workstations/order_find_workstation/OrderFindQueryDialog";
import {grey} from "@material-ui/core/colors";

type OrderFindWorkstationProps = {
    workstation: TicketWorkstationModel;
}

const useStyles = makeStyles(() => ({
    badge: {
        margin: '0px 5px',
        padding: '2px 4px',
        borderRadius: '3px',
        backgroundColor: grey[300],
        display: "inline-block"
    },
    badgeCircle: {
        height: '12px',
        width: '12px',
        borderRadius: '12px',
        display: 'inline-block',
        marginRight: '5px'
    },
    typography: {
        display: "inline-block"
    }
}));

export const OrderFindWorkstation: React.FunctionComponent<OrderFindWorkstationProps> = observer(({workstation}) => {
    const [open, handleOpen, handleClose] = useFlag();
    const [model, setModel] = useState<Model | undefined>(undefined);

    const classes = useStyles();

    const columns = useMemo(() => {
        return [
            {
                Header: t`Status`,
                accessor: (query: OrderFindQueryModel) => query,
                id: 'status',
                Cell: (cellInfo: {value: OrderFindQueryModel}) => {
                    return <Observer>
                        {
                            () => (<Grid container justifyContent={'center'}>
                                <QueryStatusBadge status={cellInfo.value.status}/>
                            </Grid>)
                        }
                    </Observer>
                },
                style: {
                    alignContent: 'center'
                },
                width: 130,
                Filter: (params: FilterParams) => {
                    return <SelectField
                        value={params.filter ? params.filter.value : ''}
                        options={[
                            { label: t`Ready`, value: 'ready' },
                            { label: t`Searching`, value: 'searching' },
                            { label: t`Running`, value: 'running' },
                            { label: t`Stopping`, value: 'stopping' }
                        ]}
                        onChange={(_, val) => {
                            params.onChange(val);
                        }}
                        clearable
                    />

                },
                filterable: true
            },
            makeColumn('name', t`Name`, 'string'),
            {
                Header: t`Results`,
                accessor: (query: OrderFindQueryModel) => query,
                id: 'results',
                Cell: (cellInfo: {value: OrderFindQueryModel}) => {
                    return <Observer>
                        {
                            () => (cellInfo.value.results ?
                                <Typography>
                                    {cellInfo.value.results + ' ' + t`Orders`}
                                </Typography> :
                                null
                            )
                        }
                    </Observer>
                },
            },
            {
                Header: t`Color`,
                accessor: (query: OrderFindQueryModel) => query,
                id: 'color',
                Cell: (cellInfo: {value: OrderFindQueryModel}) => {
                    return <Observer>
                        {
                            () => (
                                    <div className={classes.badge}>
                                        <div className={classes.badgeCircle} style={{background: `${cellInfo.value.color}`}}/>
                                        {cellInfo.value.color}
                                    </div>
                            )
                        }
                    </Observer>
                },
                Filter: (params: FilterParams) => {
                    return <SelectField
                        value={params.filter ? params.filter.value : ''}
                        options={[ //'white' | 'red' | 'green' | 'blue' | 'yellow' | 'cyan' | 'magenta'
                            { label: t`White`, value: 'white' },
                            { label: t`Red`, value: 'red' },
                            { label: t`Green`, value: 'green' },
                            { label: t`Blue`, value: 'blue' },
                            { label: t`Yellow`, value: 'yellow' },
                            { label: t`Cyan`, value: 'cyan' },
                            { label: t`Magenta`, value: 'magenta' }
                        ]}
                        onChange={(_, val) => {
                            params.onChange(val);
                        }}
                        clearable
                    />

                },
                filterable: true
            }
        ]
    }, []);

    const OrderFindWorkstationDialog = (m: Model) => {
        m.nestedAssignment('ticket_workstation_id', workstation.getId());
        return <ModelProvider value={m}>
            <OrderFindQueryDialog handleClose={handleClose}/>
        </ModelProvider>
    };

    const handleEdit = (m: Model) => {
        setModel(m);
        handleOpen();
    }

    const dialogProps =  {
        dialogTitle: t`Search Query Creation`,
            dialogForm: OrderFindWorkstationDialog,
        extraDialogProps: {
        disableEnforceFocus: true,
            style: { zIndex: 1000 }
        }
    }

    return <PaddedPaper>
        <WorkstationHelper stores={[]}>
            <ModelContext.Provider value={{model, setModel}}>
                {model && <ModelDialog
                    store={OrderFindQueryStore as unknown as DataStore<Model>}
                    open={open}
                    handleClose={handleClose}
                    dialogProps={dialogProps}
                />}
            </ModelContext.Provider>
            <ModelDialogTable
                store={OrderFindQueryStore as unknown as DataStore<Model>}
                dialogProps={dialogProps}
                columns={columns}
                createButtonProps={{
                    text: t`New Query`,
                    startIcon: <FontAwesomeIcon icon={faPlus} size={'3x'}/>
                }}
                createRightSide={true}
                tableProps={{
                    preTableActions: <WorkstationHeader
                        name={workstation.name}
                        icon={<FontAwesomeIcon icon={faSearch} size={'2x'}/>}
                    />,
                    showRefreshButton: false,
                    replacementActions: {
                        Edit: () => null,
                        Read: () => null,
                        Delete: () => null
                    },
                    customActions: (m: OrderFindQueryModel) => {
                        return <QueryActions orderQueryModel={m} handleEdit={handleEdit} handleStop={() => {}}/>
                    },
                    actionColumnWidth: 335,
                    noLinkColumns: ['status', 'name', 'results', 'color'],
                    defaultFiltered: [{id: 'ticket_workstation_id', value: workstation.getId(), persistent: true}],
                    getTdProps: (finalState: any) => {
                        return {...finalState,
                            style: {
                                alignContent: 'center'
                            }
                        }
                    }

                }}
            />
        </WorkstationHelper>
    </PaddedPaper>
})
