import * as React from "react";
import {observer} from "mobx-react-lite";
import {Clear} from "@material-ui/icons";
import {t} from "@lingui/macro";
import {TicketWorkstationModel} from "../../stores/TicketWorkstationStore";
import {AsyncBulkWorkstation} from "../../components/workstations/AsyncBulkWorkstation";
import {MultiScanField} from "../../components/workstations/async_bulk_action_workstation/base/MultiScanField";
import {BulkListItem, useBulkListItems} from "../../hooks/useBulkListItems";
import {sendEnqueueSnackbar} from "@rx-suite/ui";
import {useSnackbar} from "notistack";
import {BulkAddRecord, BulkAddTable} from "../../components/workstations/async_bulk_action_workstation/base/BulkAddTable";
import {Grid} from "@material-ui/core";
import {AsyncBulkActionResults} from "../../components/workstations/async_bulk_action_workstation/base/AsyncBulkActionResults";
import {useTransactionResults} from "../../hooks/useTransactionResults";
import {useBulkAddStatusColumns} from "../../hooks/table_columns/bulk_add/useBulkAddStatusColumns";
import {useStateColumn} from "../../hooks/table_columns/api_transactions/useStateColumn";
import {PaperlessApiTransactionStore} from "../../stores/PaperlessApiTransactionStore";
import {useEffect} from "react";
import {useClearOnNextScan} from "../../hooks/useClearOnNextScan";
import {useWorkstationCommonStyles} from "../../hooks/styles/useWorkstationCommonStyles";
import {WorkstationTableSeparator} from "../../components/workstations/WorkstationTableSeparator";

type ReleaseTicketWorkstationProps = {
    workstation: TicketWorkstationModel;
}

export const ReleaseTicketWorkstation: React.FunctionComponent<ReleaseTicketWorkstationProps> = observer(({workstation}) => {
    const {enqueueSnackbar} = useSnackbar();
    const classes = useWorkstationCommonStyles();

    const [releaseList, setReleaseList, handleFailedReleases] = useBulkListItems([], 'order_number', true);
    const [transactionIds, setTransactionIds] = useTransactionResults([]);
    const [onNextScanClear, setOnNextScanClear, clearItems] = useClearOnNextScan();

    useEffect(() => {
        // If we don't clear we will keep transactions from other workstations within the store.
        PaperlessApiTransactionStore.data.clear();
    }, []);

    const addToReleaseList = React.useCallback((scanValues: ReleaseRecord) => {
        const newReleaseList = clearItems(releaseList);

        const duplicateOrderNumber = newReleaseList.find(record => record.order_number === scanValues['order_number']);

        if(duplicateOrderNumber){
            return sendEnqueueSnackbar(t`Order Number already in list!`, {variant: "error"}, enqueueSnackbar)
        }

        setReleaseList([...newReleaseList, {order_number: scanValues['order_number'], status: null}])
    }, [releaseList, onNextScanClear]);

    const handleSubmit = async () => {
        const payload = releaseList.map((item) => {
            return {
                order_number: item.order_number
            }
        })
        try {
            const {transaction_ids, failed_values} = await workstation.callForBulkValues(payload);
            setTransactionIds(transaction_ids);
            handleFailedReleases(failed_values);
            setOnNextScanClear(true);
        }catch(err){
            sendEnqueueSnackbar(t`Failed to release tickets.`, {variant: "error"}, enqueueSnackbar)
            console.error(err);
        }
    }

    const bulkAddColumns = React.useMemo(() => {
        return [
            {
                Header: t`Order Number`,
                accessor: 'order_number',
                id: 'order_number',
                sortable: false,
                filterable: false,
                maxWidth: 225
            },
            ...useBulkAddStatusColumns()
        ]
    }, [releaseList]);

    const resultsColumns = React.useMemo(() => {
        return [
            {
                Header: t`Order Number`,
                accessor: 'associated_order_number',
                id: 'associated_order_number',
                sortable: false,
                filterable: false,
                maxWidth: 225
            },
            useStateColumn(false, true)
        ]
    }, [])

    return (
        <AsyncBulkWorkstation model={workstation} icon={<Clear fontSize={'large'}/>}>
            <MultiScanField
                fields={[{label: t`Order Number...`, key: 'order_number'}]}
                handleAllScansComplete={(data) => {addToReleaseList(data as ReleaseRecord)}}
                isLoading={workstation.isLoading}
                instructions={t`Scan an Order Number here to release it from the associated E-Ticket`}
            />
            <Grid container className={classes.tablesWrapper}>
                <BulkAddTable
                    submitText={t`Release Tickets`}
                    handleSubmit={handleSubmit}
                    isLoading={workstation.isLoading}
                    data={releaseList}
                    setData={setReleaseList as React.Dispatch<React.SetStateAction<BulkAddRecord[]>>}
                    columns={bulkAddColumns}
                />
                <WorkstationTableSeparator/>
                <AsyncBulkActionResults
                    transactionIds={transactionIds}
                    columns={resultsColumns}
                />
            </Grid>
        </AsyncBulkWorkstation>
    )
});

type ReleaseRecord = {
    order_number: string;
    status: any;
}
