import * as React from "react";
import {observer} from "mobx-react-lite";
import {sendEnqueueSnackbar, WorkstationHelper} from "@rx-suite/ui";
import {TicketWorkstationModel} from "../../stores/TicketWorkstationStore";
import {t} from "@lingui/macro";
import { FontAwesomeIcon } from "@fortawesome/react-fontawesome";
import {faAngleDoubleUp} from "@rx-suite/pro-solid-svg-icons";
import {AsyncBulkWorkstation} from "../../components/workstations/AsyncBulkWorkstation";
import {MultiScanField} from "../../components/workstations/async_bulk_action_workstation/base/MultiScanField";
import {useSnackbar} from "notistack";
import {useBulkListItems} from "../../hooks/useBulkListItems";
import {BulkAddRecord, BulkAddTable} from "../../components/workstations/async_bulk_action_workstation/base/BulkAddTable";
import {TempTicketAddInfo} from "../../hooks/table_columns/bulk_add/useBulkAddStatusColumns";
import {useClearOnNextScan} from "../../hooks/useClearOnNextScan";
import {AddStatusBadge} from "../../components/bulk_add_scan/AddStatusBadge";

type StageWorkstationProps = {
    workstation: TicketWorkstationModel;
}

export const StageTicketWorkstation: React.FunctionComponent<StageWorkstationProps> = observer(({workstation}) => {
    const {enqueueSnackbar} = useSnackbar();

    const [stageList, setStageList, handleFailedStages] = useBulkListItems([], 'ticket_barcode', true);
    const [onNextScanClear, setOnNextScanClear, clearItems] = useClearOnNextScan();

    const addToStageList = React.useCallback((scanValues: StageRecord) => {
        scanValues['ticket_barcode'] = scanValues['ticket_barcode'].toUpperCase();

        const newStageList = clearItems(stageList);

        const duplicateTicketBarcode= newStageList.find(record => record.ticket_barcode === scanValues['ticket_barcode']);

        if(duplicateTicketBarcode){
            return sendEnqueueSnackbar(t`Ticket Barcode already in list!`, {variant: "error"}, enqueueSnackbar)
        }

        setStageList([...newStageList, {ticket_barcode: scanValues['ticket_barcode'], status: null}])
    }, [stageList, onNextScanClear]);

    const bulkAddColumns = React.useMemo(() => {
        return [
            {
                Header:  t`E-Ticket Barcode`,
                accessor: 'ticket_barcode',
                id: 'ticket_barcode',
                sortable: false,
                filterable: false,
            },
            {
                Header: t`Status`,
                accessor: (entry: TempTicketAddInfo) => entry,
                id: 'status',
                filterable: false,
                Cell: (cellInfo: { value: TempTicketAddInfo }) => {
                    return (
                        <AddStatusBadge addStatus={cellInfo.value.status} successfulText={t`successfully staged`}/>
                    )
                }
            }
        ]
    }, [stageList]);

    const handleSubmit = async () => {
        const payload = stageList.map((item) => {
            return {
                ticket_barcode: item.ticket_barcode
            }
        })
        try {
            const {_, failed_values} = await workstation.callForBulkValues(payload);
            handleFailedStages(failed_values)
            setOnNextScanClear(true);
        }catch(err){
            sendEnqueueSnackbar(t`Failed to stage tickets.`, {variant: "error"}, enqueueSnackbar)
            console.error(err);
        }
    }

    return (
        <WorkstationHelper stores={[]}>
            <AsyncBulkWorkstation model={workstation} icon={<FontAwesomeIcon icon={faAngleDoubleUp} size={'3x'}/>}>
                <MultiScanField
                    fields={[
                        {
                            label: t`E-Ticket Barcode`,
                            key: 'ticket_barcode',
                        }]}
                    handleAllScansComplete={(data) => {addToStageList(data as StageRecord)}}
                    isLoading={workstation.isLoading}
                    instructions={t`Scan an E-Ticket barcode here to stage it`}
                />
                <BulkAddTable
                    submitText={t`Stage Tickets`}
                    handleSubmit={handleSubmit}
                    isLoading={workstation.isLoading}
                    data={stageList}
                    setData={setStageList as React.Dispatch<React.SetStateAction<BulkAddRecord[]>>}
                    columns={bulkAddColumns}
                />
            </AsyncBulkWorkstation>
        </WorkstationHelper>
    );
})

type StageRecord = {
    ticket_barcode: string;
    status: any;
}

