import * as React from "react";
import {TicketWorkstationModel} from "../../stores/TicketWorkstationStore";
import {Link} from "@material-ui/icons";
import {Grid} from "@material-ui/core";
import {observer} from "mobx-react-lite";
import {AsyncBulkWorkstation} from "../../components/workstations/AsyncBulkWorkstation";
import {MultiScanField} from "../../components/workstations/async_bulk_action_workstation/base/MultiScanField";
import {t} from "@lingui/macro";
import {sendEnqueueSnackbar} from "@rx-suite/ui";
import {useSnackbar} from "notistack";
import {BulkAddRecord, BulkAddTable} from "../../components/workstations/async_bulk_action_workstation/base/BulkAddTable";
import {useBulkAddStatusColumns} from "../../hooks/table_columns/bulk_add/useBulkAddStatusColumns";
import {useStateColumn} from "../../hooks/table_columns/api_transactions/useStateColumn";
import {PaperlessApiTransactionStore} from "../../stores/PaperlessApiTransactionStore";
import {useTransactionResults} from "../../hooks/useTransactionResults";
import {useBulkListItems} from "../../hooks/useBulkListItems";
import {AsyncBulkActionResults} from "../../components/workstations/async_bulk_action_workstation/base/AsyncBulkActionResults";
import {useEffect} from "react";
import {useClearOnNextScan} from "../../hooks/useClearOnNextScan";
import {useWorkstationCommonStyles} from "../../hooks/styles/useWorkstationCommonStyles";
import {WorkstationTableSeparator} from "../../components/workstations/WorkstationTableSeparator";

type TicketAssociationWorkstationProps = {
    workstation: TicketWorkstationModel;
}

export const TicketAssociationWorkstation: React.FunctionComponent<TicketAssociationWorkstationProps> = observer(({workstation}) => {
    const classes = useWorkstationCommonStyles();
    const {enqueueSnackbar} = useSnackbar();

    const [associationList, setAssociationList, handleFailedAssociations] = useBulkListItems([], 'launch_number', true);
    const [transactionIds, setTransactionIds] = useTransactionResults([]);
    const [onNextScanClear, setOnNextScanClear, clearItems] = useClearOnNextScan();

    useEffect(() => {
        // If we don't clear we will keep transactions from other workstations within the store.
        PaperlessApiTransactionStore.data.clear();
    }, []);

    const bulkAddColumns = React.useMemo(() => {
        return [
            {
                Header:  t`Launch Number`,
                accessor: 'launch_number',
                id: 'order_number',
                sortable: false,
                filterable: false,
                maxWidth: 225
            },
            {
                Header:  t`E-Ticket Barcode`,
                accessor: 'ticket_barcode',
                id: 'ticket_barcode',
                sortable: false,
                filterable: false,
                maxWidth: 225
            },
            ...useBulkAddStatusColumns()
        ]
    }, [associationList]);

    const resultsColumns = React.useMemo(() => {
        return [
            {
                Header: t`Launch Number`,
                accessor: 'associated_order_launch_number',
                id: 'associated_order_launch_number',
                sortable: false,
                filterable: false,
                maxWidth: 225
            },
            {
                Header: t`E-Ticket Barcode`,
                accessor: 'associated_ticket_barcode',
                id: 'associated_ticket_barcode',
                sortable: false,
                filterable: false,
                maxWidth: 225
            },
            useStateColumn(false, true)
        ]
    }, [])

    const addToAssociationList = React.useCallback((scanValues: AssociationRecord) => {
        const newAssociationList = clearItems(associationList);

        const duplicateLaunchNumber = newAssociationList.find(record => record.launch_number === scanValues['launch_number']);

        if(duplicateLaunchNumber){
            return sendEnqueueSnackbar(t`Launch Number already in list!`, {variant: "error"}, enqueueSnackbar)
        }

        scanValues['ticket_barcode'] = scanValues['ticket_barcode'].toUpperCase();

        const duplicateTicketBarcode = newAssociationList.find(record => record.ticket_barcode === scanValues['ticket_barcode']);

        if(duplicateTicketBarcode){
            return sendEnqueueSnackbar(t`Ticket Barcode already in list!`, {variant: "error"}, enqueueSnackbar)
        }

        setAssociationList([...newAssociationList, {ticket_barcode: scanValues['ticket_barcode'], launch_number: scanValues['launch_number'], status: ''}])
    }, [associationList, onNextScanClear]);

    const handleSubmit = async () => {
        // Clear old status
        setAssociationList((prevState) => {
            const cleared_status = prevState.map((association) => ({...association, status: ''}));
            return [...cleared_status];
        })

        const payload = associationList.map((item) => {
            return {
                launch_number: item.launch_number,
                ticket_barcode: item.ticket_barcode
            }
        })
        try {
            const {transaction_ids, failed_values} = await workstation.callForBulkValues(payload);
            setTransactionIds(transaction_ids);
            handleFailedAssociations(failed_values);
            setOnNextScanClear(true);
        }catch(err){
            sendEnqueueSnackbar(t`Failed to associate tickets.`, {variant: "error"}, enqueueSnackbar)
            console.error(err);
        }
    }

    return (
        <AsyncBulkWorkstation model={workstation} icon={<Link fontSize={'large'}/>}>
            <MultiScanField fields={[
                    {
                        label: t`Launch Number`,
                        key: 'launch_number',
                    },
                    {
                        label: t`E-ticket Barcode`,
                        key: 'ticket_barcode'
                    }
                ]}
                handleAllScansComplete={(data) => {addToAssociationList(data as AssociationRecord)}}
                isLoading={workstation.isLoading}
                instructions={t`Scan an Order Number and E-Ticket Barcode here to associate the order to the E-Ticket.`}
            />
            <Grid container className={classes.tablesWrapper}>
                <BulkAddTable
                    submitText={t`Associate Tickets`}
                    handleSubmit={handleSubmit}
                    isLoading={workstation.isLoading}
                    data={associationList}
                    setData={setAssociationList as React.Dispatch<React.SetStateAction<BulkAddRecord[]>>}
                    columns={bulkAddColumns}
                />
                <WorkstationTableSeparator/>
                <AsyncBulkActionResults
                    transactionIds={transactionIds}
                    columns={resultsColumns}
                />
            </Grid>
        </AsyncBulkWorkstation>
    )
})

type AssociationRecord = {
    ticket_barcode: string;
    launch_number: string;
    status: any;
}
