import * as React from "react";
import {observer} from "mobx-react-lite";
import {Web} from "@material-ui/icons";
import {t} from "@lingui/macro";
import {TicketWorkstationModel} from "../../stores/TicketWorkstationStore";
import {AsyncBulkWorkstation} from "../../components/workstations/AsyncBulkWorkstation";
import {MultiScanField} from "../../components/workstations/async_bulk_action_workstation/base/MultiScanField";
import {sendEnqueueSnackbar} from "@rx-suite/ui";
import {useSnackbar} from "notistack";
import {useBulkListItems} from "../../hooks/useBulkListItems";
import {useTransactionResults} from "../../hooks/useTransactionResults";
import {useClearOnNextScan} from "../../hooks/useClearOnNextScan";
import {Grid} from "@material-ui/core";
import {BulkAddRecord, BulkAddTable} from "../../components/workstations/async_bulk_action_workstation/base/BulkAddTable";
import {AsyncBulkActionResults} from "../../components/workstations/async_bulk_action_workstation/base/AsyncBulkActionResults";
import {useBulkAddStatusColumns} from "../../hooks/table_columns/bulk_add/useBulkAddStatusColumns";
import {PaperlessApiTransactionStore} from "../../stores/PaperlessApiTransactionStore";
import {useStateColumn} from "../../hooks/table_columns/api_transactions/useStateColumn";
import {useEffect} from "react";
import {useWorkstationCommonStyles} from "../../hooks/styles/useWorkstationCommonStyles";
import {WorkstationTableSeparator} from "../../components/workstations/WorkstationTableSeparator";

type WriteTemplateWorkstationProps = {
    workstation: TicketWorkstationModel;
}

export const WriteTemplateWorkstation: React.FunctionComponent<WriteTemplateWorkstationProps> = observer(({workstation}) => {
    const classes = useWorkstationCommonStyles();
    const {enqueueSnackbar} = useSnackbar();

    const [writeList, setWriteList, handleFailedAssociations] = useBulkListItems([], 'order_number', true);
    const [transactionIds, setTransactionIds] = useTransactionResults([]);
    const [onNextScanClear, setOnNextScanClear, clearItems] = useClearOnNextScan();

    useEffect(() => {
        // If we don't clear we will keep transactions from other workstations within the store.
        PaperlessApiTransactionStore.data.clear();
    }, []);

    const addToAssociationList = React.useCallback((scanValues: WriteRecord) => {
        const newWriteList = clearItems(writeList);

        const duplicateOrderNumber = newWriteList.find(record => record.order_number === scanValues['order_number']);

        if(duplicateOrderNumber){
            return sendEnqueueSnackbar(t`Order Number already in list!`, {variant: "error"}, enqueueSnackbar)
        }

        setWriteList([...newWriteList, {order_number: scanValues['order_number'], status: null}])
    }, [writeList, onNextScanClear]);

    const bulkAddColumns = React.useMemo(() => {
        return [
            {
                Header: t`Order Number`,
                accessor: 'order_number',
                id: 'order_number',
                sortable: false,
                filterable: false,
                maxWidth: 225
            },
            ...useBulkAddStatusColumns()
        ]
    }, [writeList]);

    const resultsColumns = React.useMemo(() => {
        return [
            {
                Header: t`Order Number`,
                accessor: 'associated_order_number',
                id: 'associated_order_number',
                sortable: false,
                filterable: false,
                maxWidth: 225
            },
            useStateColumn(false, true)
        ]
    }, [])

    const handleSubmit = async () => {
        const payload = writeList.map((item) => {
            return {
                order_number: item.order_number,
            }
        })
        try {
            const {transaction_ids, failed_values} = await workstation.callForBulkValues(payload);
            setTransactionIds(transaction_ids);
            handleFailedAssociations(failed_values);
            setOnNextScanClear(true);
        }catch(err){
            sendEnqueueSnackbar(t`Failed to write tickets.`, {variant: "error"}, enqueueSnackbar)
            console.error(err);
        }
    }

    return <AsyncBulkWorkstation model={workstation} icon={<Web fontSize={'large'}/>}>
        <MultiScanField fields={[
            {
                label: t`Order Number...`,
                key: 'order_number'
            }]}
            handleAllScansComplete={(data) => {addToAssociationList(data as WriteRecord)}}
            isLoading={workstation.isLoading}
            instructions={t`Scan an Order Number here to write the template to the E-Ticket`}
        />
        <Grid container className={classes.tablesWrapper}>
            <BulkAddTable
                submitText={t`Write Tickets`}
                handleSubmit={handleSubmit}
                isLoading={workstation.isLoading}
                data={writeList}
                setData={setWriteList as React.Dispatch<React.SetStateAction<BulkAddRecord[]>>}
                columns={bulkAddColumns}
            />
            <WorkstationTableSeparator/>
            <AsyncBulkActionResults
                transactionIds={transactionIds}
                columns={resultsColumns}
            />
        </Grid>
    </AsyncBulkWorkstation>
});

type WriteRecord = {
    ticket_barcode: string;
    order_number: string;
    status: any;
}
