import React from "react";
import {dataStorePermission, PasswordResetForm} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {
    PostAdd,
    ViewQuilt,
    Settings,
    Send,
    PlayCircleFilled,
    Event,
    Web,
    Clear, Devices, Label,
    Link, AccountTree
} from '@material-ui/icons';
import {SettingForm} from "./pages/settings/SettingForm";
import {SettingStore} from "./stores/SettingStore";
import {OrderRecordStore} from "./stores/OrderRecordStore";
import {PaperlessTicketStore} from "./stores/PaperlessTicketStore";
import {PaperlessTicketBreadModel} from "./pages/paperless_tickets/PaperlessTicketBreadModel";
import {PaperlessApiTransactionsBreadModel} from "./pages/paperless_api_transactions/PaperlessApiTransactionsBreadModel";
import {PaperlessApiTransactionStore} from "./stores/PaperlessApiTransactionStore";
import {LaunchingOrderRecordsBreadModel} from "./pages/order_records/bread_models/LaunchingOrderRecordsBreadModel";
import {LiveOrderRecordsBreadModel} from "./pages/order_records/bread_models/LiveOrderRecordsBreadModel";
import {TicketTemplateStore} from "./stores/TicketTemplateStore";
import {TicketTemplateBreadModel} from "./pages/ticket_templates/TicketTemplateBreadModel";
import {computedFn} from "mobx-utils";
import {faAngleDoubleUp, faFire, faGlasses, faSearch, faTachometerAltFast} from "@rx-suite/pro-solid-svg-icons";
import {FontAwesomeIcon} from "@fortawesome/react-fontawesome";
import {TicketWorkstationModel, TicketWorkstationStore} from "./stores/TicketWorkstationStore";
import {WorkstationBreadModel} from "./pages/workstation_settings/WorkstationBreadModel";
import {TagStore} from "./stores/TagStore";
import {TagBreadModel} from "./pages/tags/TagBreadModel";
import { WriteTemplateWorkstation } from "./pages/workstations/WriteTemplateWorkstation";
import {StageTicketWorkstation} from "./pages/workstations/StageTicketWorkstation";
import {ReleaseTicketWorkstation} from "./pages/workstations/ReleaseTicketWorkstation";
import { TicketAssociationWorkstation } from "./pages/workstations/TicketAssocciationWorkstation";
import {WorkflowStore} from "./stores/WorkflowStore";
import {WorkflowBreadModel} from "./pages/workflows/WorkflowBreadModel";
import {currentUserCan} from "@rx-suite/stores";
import {TicketInWorkflowView} from "./pages/workflows/TicketInWorkflowView";
import {MarkHotJobWorkstation} from "./pages/workstations/MarkHotJobWorkstation";
import {performanceMetrics} from "./pages/performance_metrics/PerformanceMetrics";
import {OrderFindWorkstation} from "./pages/workstations/OrderFindWorkstation";

const faIconStyle = {height: '22px', width: '22px'};

const resolveWorkstation = (workstation: TicketWorkstationModel) => {

    const route = {
        label: workstation.name,
        to: `/${workstation.id}`,
        permission: {
            target: `paperless_ticket_workstation_${workstation.id}`,
            action: 'use'
        },
        exact: false,
        Icon: null,
        Component: null,
        shouldMount: () => true
    }

    switch(workstation.action){
        case 'write_template':
            route.Icon = Web;
            route.Component = () => <WriteTemplateWorkstation workstation={workstation}/>
            break;
        case 'stage':
            route.Icon = () => <FontAwesomeIcon style={faIconStyle} icon={faAngleDoubleUp}/>
            route.Component = () =>  <StageTicketWorkstation workstation={workstation}/>
            route.shouldMount = () => !SettingStore.selectedModel?.isManualPairingMode
            break;
        case 'release':
            route.Icon = Clear
            route.Component = () => <ReleaseTicketWorkstation workstation={workstation}/>
            break;
        case 'associate':
            route.Icon = Link
            route.Component = () => <TicketAssociationWorkstation workstation={workstation}/>
            route.shouldMount = () => !!SettingStore.selectedModel?.isManualPairingMode
            break;
        case 'mark_hot_job':
            route.Icon = () => <FontAwesomeIcon style={faIconStyle} icon={faFire}/>
            route.Component = () => <MarkHotJobWorkstation workstation={workstation}/>
            break;
        case "order_find":
            route.Icon = () => <FontAwesomeIcon style={faIconStyle} icon={faSearch}/>
            route.Component = () => <OrderFindWorkstation workstation={workstation}/>
            break;
    }

    return route;
}

// Create a clone of the WS store so that preloading the models form the menu does not break the BREAD model for edits
const workstationStore = TicketWorkstationStore.clone();

export const getRoutes = computedFn(() => {
    return [
        {
            label: '',
            to: "/",
            exact: true,
            Component: LiveOrderRecordsBreadModel
        },
        {
            label: t`Workstations`,
            to: "/workstations",
            exact: false,
            Icon: Devices,
            children: (workstationStore.getAll() as TicketWorkstationModel[])
                .sort((a,b) => a.display_order - b.display_order)
                .map( workstation => (
                    resolveWorkstation(workstation)
                )
            )
        },
        {
            label: t`Orders`,
            to: "",
            exact: false,
            permission: dataStorePermission(OrderRecordStore),
            Icon: () => <FontAwesomeIcon icon={faGlasses} style={faIconStyle}/>,
            children: [
                {
                    label: t`In Progress`,
                    to: "/orders",
                    exact: false,
                    Component: LiveOrderRecordsBreadModel,
                    Icon: PlayCircleFilled
                },
                {
                    label: t`Waiting Launch`,
                    to: "/launch_queue",
                    exact: true,
                    Component: LaunchingOrderRecordsBreadModel,
                    Icon: Send
                }
            ]
        },
        {
            label: "Tickets",
            to: "/paperless_tickets",
            exact: false,
            permission: dataStorePermission(PaperlessTicketStore),
            Component: PaperlessTicketBreadModel,
            Icon: PostAdd
        },
        {
            label: t`Workflows`,
            to: "/workflows",
            exact: false,
            Icon: AccountTree,
            Component: TicketInWorkflowView,
            shouldMount: () => currentUserCan('Workflow', 'read')
        },
        {
            label: "Event Log",
            to: "/events",
            exact: false,
            permission: dataStorePermission(PaperlessApiTransactionStore),
            Component: PaperlessApiTransactionsBreadModel,
            Icon: Event
        },
        {
            label: t`Performance`,
            to: "/performance_metrics",
            exact: false,
            Icon: () => <FontAwesomeIcon style={faIconStyle} icon={faTachometerAltFast}/>,
            Component: performanceMetrics,
            shouldMount: () => currentUserCan('PerformanceMetrics', 'view')
        },
        {
            label: "Settings",
            to: "/settings",
            exact: true,
            permission: dataStorePermission(SettingStore),
            Component: SettingForm,
            Icon: Settings,
            children: [
                {
                    label: t`Ticket Templates`,
                    to: "/ticket_templates",
                    exact: false,
                    permission: dataStorePermission(TicketTemplateStore),
                    Component: TicketTemplateBreadModel,
                    Icon: ViewQuilt
                },
                {
                    label: t`Tags`,
                    to: "/tags",
                    exact: false,
                    permission: dataStorePermission(TagStore),
                    Component: TagBreadModel,
                    Icon: Label
                },
                {
                    label: t`Workstations`,
                    to: "/workstations",
                    exact: false,
                    permission: dataStorePermission(TicketWorkstationStore),
                    Component: WorkstationBreadModel,
                    Icon: Web
                },
                {
                    label: t`Workflows`,
                    to: "/workflows",
                    exact: false,
                    permission: dataStorePermission(WorkflowStore),
                    Component: WorkflowBreadModel,
                    Icon: AccountTree
                }
            ]
        },
        {
            to: "/password_reset/:user/:token",
            exact: false,
            Component: PasswordResetForm,
        }
    ];
})