import {attribute, DataStore, errToString, Model, model} from "@rx-suite/data";
import {t} from "@lingui/macro";
import * as yup from "yup";
import {LedBlinkSpeedOption, LedColorOption} from "./SettingStore";
import {JsonLogicResult} from "@react-awesome-query-builder/antd";
import {action, computed, runInAction} from "mobx";

export type queryStatus = "ready" | "searching" | "running" | "stopping";

@model({
    transportLayer: "/api/order_find_queries",
    permissionTarget: "OrderFindQuery",
    routingInfo: {
        create: '/order_find_queries/create',
        read: '/order_find_queries/:id/read',
        update: '/order_find_queries/:id/edit'
    }
})
export class OrderFindQueryModel extends Model {
    @attribute({
        label: t`Name`,
        validation: yup.string().required(t`Name is a required field`)
    })
    name: string = "";

    @attribute({
        label: t`Status`,
        validation: yup.string()
    })
    status: queryStatus = "ready";

    @attribute({
        label: t`Color`,
        validation: yup.string().required(t`Color is a required field`)
    })
    color: LedColorOption = "red";

    @attribute({
        label: t`Blink Pattern`,
        validation: yup.string().required(t`Blink Pattern is a required field`)
    })
    blink_pattern: LedBlinkSpeedOption = "FLASH_1_SECOND"

    @attribute({
        label: t`Blink Duration (minutes)`,
        validation: yup.number().nullable()
    })
    blink_duration: undefined | number = undefined;

    @attribute({
        label: t`Results`,
        validation: yup.number().nullable()
    })
    results: number | undefined = undefined;

    @attribute({
        label: t`Query`,
        validation: yup.object()
    })
    query: JsonLogicResult | undefined = undefined;

    @attribute()
    ticket_workstation_id: number | undefined = undefined

    static types = {
        query: 'json'
    }

    @computed
    get canExecute() {
        return this.status === 'ready';
    }

    @computed
    get canStop() {
        return this.status === 'running';
    }

    @action
    async execute() {
        this.startModelOperation();

        try {
            const resp = await this.store.transportLayer.fetch(`/${this.id}/execute`, {
                method: "PUT",
                headers: {"Content-Type": "application/json"}
            })
            return resp.json();
        } catch(e) {
            console.warn(errToString(e));
            return {error: true};
        } finally {
            this.stopModelOperation();
        }
    }

    @action
    async stop() {
        this.startModelOperation();

        try {
            const resp = await this.store.transportLayer.fetch(`/${this.id}/stop`, {
                method: "PUT",
                headers: {"Content-Type": "application/json"}
            })
            return resp.json();
        } catch(e) {
            console.warn(errToString(e));
            return {error: true};
        } finally {
            this.stopModelOperation();
        }
    }

    @action
    startModelOperation() {
        runInAction(() => {
            this._state = 'updating';
        })
    }

    @action
    stopModelOperation() {
        runInAction(() => {
            this._state = 'done';
        })
    }

    constructor() {
        super();
        super.setup();
    }
}
export const OrderFindQueryStore = OrderFindQueryModel.store as DataStore<OrderFindQueryModel>;