import {attribute, DataStore, model, Model} from "@rx-suite/data";
import * as yup from 'yup';
import {t} from "@lingui/macro";
import FileSaver from 'file-saver'
import {action, computed} from "mobx";
import {PaperlessApiTransactionStore} from "./PaperlessApiTransactionStore";

export type OrderRecordState = 'waiting_for_launch' | 'failed_launch' | 'launching' | 'launched';

type OrderRecordFetchError = { fetchError: string };

export enum Side {
    LEFT = 'left',
    RIGHT = 'right'
}

export type OrderRecordDataResponse = {
    [key in Side]: OrderRecordSideData | null;
};

type OrderRecordSideData = {
    [name: string]: string | null;
};

@model({
    transportLayer: "/api/order_records",
    permissionTarget: "OrderRecord",
    routingInfo: {
        read: "/orders/:id/read"
    }
})

export class OrderRecordModel extends Model {
    constructor() {
        super();
        super.setup();
    }

    @attribute({
        label: t`Job Number`,
        validation: yup.string().required(),
        autoForm: false
    })
    number: string = '';

    @attribute({
        label: t`Launch Number`,
        validation: yup.string().required(),
        autoForm: false
    })
    launch_number: string = '';

    @attribute({
        label: t`Entry Time`,
        validation: yup.date().required(),
        autoForm: false
    })
    entry_time: Date = new Date();

    @attribute({ label: t`State` })
    state: OrderRecordState = 'waiting_for_launch';

    @attribute({
        label: t`Last Action Time`,
        validation: yup.date().required(),
        autoForm: false
    })
    last_action_time: Date = new Date();

    @attribute({ label: t`Failed Reason` })
    failed_reason: string | null = null;

    @attribute({ label: t`Rx-Data Order Id`})
    rx_data_order_id: number | null = null;

    @attribute({ label: t`Is Hot Job?`})
    is_hot_job: boolean = false;

    @attribute({ label: t`Download Link`})
    download_link: string | null = null;

    @attribute({ label: t`Created At` })
    created_at: Date = new Date();

    @attribute({ label: t`Updated At` })
    updated_at: Date = new Date();

    @attribute()
    paperless_ticket_id: number | null = null;

    @attribute({ label: t`Current Template`})
    ticket_template_id: number | null = null;

    @attribute({ label: t`E-Ticket Barcode`})
    paperless_ticket_barcode: string | null = null;

    @attribute()
    hot_job_toggling: boolean = false;

    @computed
    get isAttachedToTicket() {
        return this.paperless_ticket_id !== null;
    }

    @action.bound
    downloadPdf = () => {
        if (this.download_link) {
            FileSaver.saveAs(this.download_link)
        }
    };

    @action.bound
    viewPdf = async () => {
        if(!this.download_link){
            throw 'No download link available';
        }

        const response = await OrderRecordStore.transportLayer.fetch(`/${this.id}/download`, {
            method: "GET"
        });

        const blob = await response.blob();

        if (!blob) {
            throw 'Failed to download PDF blob';
        }

        // TODO: need to use revokeObjectURL(blob) later on
        const fileURL = URL.createObjectURL(blob);

        const pdfWindow = window.open();
        if (!pdfWindow) {
            throw 'Unable to open new window';
        }

        pdfWindow.location.href = fileURL;
    };

    @action.bound
    toggleHotJob = async () => {
        const response = await OrderRecordStore.transportLayer.fetch(`/${this.id}/hot_job`, {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify({}) // no params needed
        });

        const data = await response.json();

        if(data.error){
            throw data.error;
        }

        PaperlessApiTransactionStore.getTableData(true);

        return data;
    };

    @action.bound
    findMyJob = async () => {
        const response = await OrderRecordStore.transportLayer.fetch(`/${this.id}/find_my_job`, {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify({}) // no params needed
        });

        const data = await response.json();

        if (data.error) {
            throw data.error;
        }

        PaperlessApiTransactionStore.getTableData(true);

        return data;
    };

    @action.bound
    launchJob = async () => {
        // By-pass the store transport so we can handle the error ourselves
        const response = await fetch(`${OrderRecordStore.transportLayer.base}/${this.id}/launch_job`, {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify({}) // no params needed
        });

        const data = await response.json();

        if(!response.ok){
            throw data.code || data.error;
        }

        return data;
    };

    @action.bound
    changeTemplate = async (templateId: number) => {
        const response = await this.store.transportLayer.fetch(`/${this.id}/change_template`, {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify({template_id: templateId})
        })

        const data = await response.json();

        return data;
    }

    @action.bound
    fetchTags = async (): Promise<Readonly<OrderRecordDataResponse>> => {
        const response = await OrderRecordStore.transportLayer.fetch(`/${this.id}/tags`, {
            method: "GET"
        });

        const data = await response.json();

        if (data.error) {
            throw data.error;
        }

        return data.tags;
    };

    @action.bound
    latestLaunchTransaction_id = async (): Promise<number> => {
        const response = await OrderRecordStore.transportLayer.fetch(`/${this.id}/latest_launch_transaction_id`, {
            method: "GET"
        });

        const data = await response.json();

        if (data.error) {
            throw data.error;
        }

        return data.transaction_id;
    };

    canDelete(): boolean {
        return this.state === 'waiting_for_launch' || this.state === 'failed_launch';
    }
}

export const fetchByJobNumber = async (jobNumber: string): Promise<OrderRecordModel | OrderRecordFetchError> => {

    const response = await OrderRecordStore.transportLayer.fetch(`/fetch_by_job_number`, {
        method: "POST",
        headers: {
            "Content-Type": "application/json"
        },
        body: JSON.stringify({
            number: jobNumber
        })
    });

    const respJson = await response.json();

    if ('error' in respJson) {
        return {
            fetchError: respJson['error']
        };
    }

    return respJson;
};

export const OrderRecordStore = OrderRecordModel.store as DataStore<OrderRecordModel>;
