import {attribute, DataStore, Model, model} from '@rx-suite/data';
import { t } from "@lingui/macro"
import {action, computed} from "mobx";
import {TicketTemplateStore} from "./TicketTemplateStore";

export const paperlessApiTransactionStates = ['queued', 'ongoing', 'completed', 'failed', 'canceled'];
export type PaperlessApiTransactionState = typeof paperlessApiTransactionStates[number];

export const paperlessApiTransactionTypes = ["launch", "set_hot", "clear_hot", "release", "set_template", "find", "stop_find", "unknown"];
export type PaperlessApiTransactionType = typeof paperlessApiTransactionTypes[number];

@model({
    transportLayer: "/api/paperless_api_transactions",
    permissionTarget: "PaperlessApiTransaction",
    routingInfo: {
        list: "/events",
        read: "/events/:id/read",
    }
})
export class PaperlessApiTransactionModel extends Model {

    constructor() {
        super();
        super.setup();
    }

    @attribute({label: t`State`})
    state: PaperlessApiTransactionState = 'queued';

    @attribute({label: t`Transaction Identifier`})
    transaction_identifier: string = "";

    @attribute({label: t`Transaction Type`})
    transaction_type: PaperlessApiTransactionType = "unknown";

    @attribute({label: t`Request Body`})
    request_body: string | null = null;

    @attribute({label: t`Response Body`})
    response_body: string | null = null;

    @attribute({label: t`Source`})
    source: string = 'SYSTEM';

    @attribute({label: t`Created At`})
    created_at: string = '';

    @attribute({label: t`Finished At`})
    finished_at: string = '';

    @attribute({label: t`Duration`})
    duration: number = 0;

    // Join model data that is always present

    @attribute()
    associated_ticket_barcode: string = '';

    @attribute()
    associated_order_number: string = '';

    @attribute()
    associated_order_launch_number: string = '';

    @attribute({})
    ticket_template_id: null | number = null;

    @computed
    get isComplete() {
        return this.state === 'completed';
    }

    @computed
    get isCanceled() {
        return this.state === 'canceled';
    }

    @computed
    get isFailed() {
        return this.state === 'failed';
    }

    @computed
    get isProcessed() {
        return this.isComplete || this.isFailed || this.isCanceled;
    }

    @computed
    get transaction_error() {
        if(this.state !== "failed" || this.response_body === null){
            return null;
        }

        const body = JSON.parse(this.response_body);

        if(!body["error"] && !body["message"]){
            return null;
        }

        return body["error"] || body["message"];
    }

    @computed
    get resolved_transaction_type(){
        if(this.transaction_type === 'set_template'){
            const templateModel = TicketTemplateStore.models.find(t => t.id === this.ticket_template_id);

            if (templateModel == null) {
                return "Set Template";
            }else{
                return `Set Template (${templateModel.process_name})`;
            }
        }

        const type = {
            launch: "Launch",
            set_hot: "Set Hot",
            clear_hot: "Clear Hot",
            release: "Release",
            find: "Find",
            stop_find: "Stop Blink"
        }[this.transaction_type as 'launch' | 'set_hot' | 'clear_hot' | 'release' | 'find' | 'stop_find']

        if(!type){
            return 'unknown'
        }

        return type;
    }

    @action.bound
    refreshStatus = async () => {
        const response = await PaperlessApiTransactionStore.transportLayer.fetch(`/${this.id}/status`, {
            method: "GET"
        });

        const data = await response.json();

        if (data.error) {
            throw data.error;
        }

        return data;
    };

    @action
    cancelTransaction = async () => {
        const response = await PaperlessApiTransactionStore.transportLayer.fetch(`/${this.id}/cancel_transaction`, {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        })

        await response.json();
    }
}

export const PaperlessApiTransactionStore = PaperlessApiTransactionModel.store as DataStore<PaperlessApiTransactionModel>;
