import {attribute, DataStore, Model, model} from '@rx-suite/data';
import { t } from "@lingui/macro"
import {action, computed, makeObservable, observable, runInAction} from "mobx";
import {TicketStaticDatumModel} from "./TicketStaticDatumStore";

type TicketTypeFilter = TicketState | 'all';

type TicketCountData = {
    available: number;
    staged: number;
    in_use: number;
    total: number;
    [key: string]: number;
}

export type TicketState = 'available' | 'staged' | 'in_use';

class PaperlessTicketStoreDataStore<T extends Model> extends DataStore<T> {
    @observable ticketStateFilter: TicketTypeFilter = "available";

    @observable fetchingStats: boolean = false;

    @observable ticketCounts: TicketCountData = {
            available: 0,
            staged: 0,
            in_use: 0,
            total: 0
    }

    @action
    async refreshTicketCounts() {
        if(this.fetchingStats){
            return;
        }

        this.fetchingStats = true;

        const response = await this.transportLayer.fetch(`/count`);

        const data = await response.json();

        if(data.counts){
            this.ticketCounts = data.counts;
        }

        this.fetchingStats = false;
    }

    async findByBarcode(barcode: string) {
        const response = await this.transportLayer.fetch(`/find_by_barcode`, {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify({
                ticket_barcode: barcode
            })
        })

        const data = await response.json();

        if(data.not_found){
            return undefined;
        }

        const model = this.createModel(data);
        return this.setModel(model);
    }

    async bulkAddTickets(tickets: Record<string, string>[]) {
        this.fetching = true;
        const response = await this.transportLayer.fetch('/bulk_add', {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            },
            body: JSON.stringify({tickets: tickets})
        })
        this.fetching = false;
        return await response.json();
    }

    async changeTicketsShown(state_filter: TicketTypeFilter) {
        if (this.fetching) {
            return;
        }
        runInAction(() => {
            this.ticketStateFilter = state_filter
        });
        const filter = this.fetchParams.filtered.find(filter => filter.id === 'state');
        if(!filter){
            this.fetchParams.filtered.push({id: 'state', value: this.ticketStateFilter});
        } else {
            filter.value = this.ticketStateFilter
        }
        this.getTableData(true);
    }

    constructor(model, transportLayer, permissionTarget, routingInfo, eager) {
        super(model, transportLayer, permissionTarget, routingInfo, eager);
        makeObservable(this);
    }
}

@model({
    transportLayer: "/api/paperless_tickets",
    permissionTarget: "PaperlessTicket",
    routingInfo: {
        list: "/paperless_tickets",
        read: "/paperless_tickets/:id/read",
    },
    storeClass: PaperlessTicketStoreDataStore
})
export class PaperlessTicketModel extends Model {

    @attribute({label: t`Ticket Barcode`})
    ticket_barcode: string = "";

    @attribute({label: t`Order Number`})
    order_number: string | null = null;

    @attribute()
    state: TicketState = 'available';

    @attribute()
    releasing: boolean = false;

    @attribute()
    new_static_data: TicketStaticDatumModel[] = [];

    canDelete(...args: any[]) {
        return !(this.isAttached || this.isStaged)
    }

    @computed
    get isStaged(){
        return this.state === 'staged';
    }

    @computed
    get isAttached(){
        return this.order_number !== null;
    }

    @action
    async release() {
        const response = await this.store.transportLayer.fetch(`/${this.id}/release`, {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        });

        await response.json();
    }

    @action
    async toggleStaged() {
        const response = await this.store.transportLayer.fetch(`/${this.id}/toggle_staged`, {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        });

        await response.json();
    }

    @action
    async forceStopBlink() {
        return await this.store.transportLayer.fetch(`/${this.id}/force_stop_blink`, {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        });
    }

    @action.bound
    fetchImage = async (): Promise<string> => {
        const response = await PaperlessTicketStore.transportLayer.fetch(`/${this.id}/current_image`, {
            method: "GET"
        });

        const blob = await response.blob();

        if (!blob) {
            throw 'Failed to download PDF blob';
        }

        // TODO: need to use revokeObjectURL(blob) later on
        return URL.createObjectURL(blob);
    }

    constructor() {
        super();
        super.setup();
    }
}

export const PaperlessTicketStore = PaperlessTicketModel.store as PaperlessTicketStoreDataStore<PaperlessTicketModel>;
