import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import { t } from "@lingui/macro"
import {computed} from "mobx";
import {string} from "yup";
import {SelectOption} from "react-select-material-ui";

export type LedColorOption = 'white' | 'red' | 'green' | 'blue' | 'yellow' | 'cyan' | 'magenta'
export type LedBlinkSpeedOption = 'FLASH_1_SECOND' | 'FLASH_2_SECOND' | 'FLASH_4_SECOND' | 'FLASH_MULTI'

@model({
    transportLayer: "/api/settings",
    permissionTarget: "Setting",
    singleton: true,
    routingInfo: {
        update: '/settings'
    }
})

export class SettingModel extends Model {
    @attribute()
    id: number = 1;

    @attribute({
        label: t`LMS connection`,
        validation: yup.number().typeError(t`Select a valid LMS connection.`)
    })
    external_dcs_id: number | null = null

    @attribute({
        label: t`Order Number VCA Tag`,
        validation: yup.string()
    })
    order_number_tag: string = 'JOB'

    @attribute({
        label: t`Job Folder Path`,
        validation: string().when('isManualPairingMode',{
            is: true,
            then: schema => schema.nullable(),
            otherwise: schema => schema.required(t`Path to Order Folder is a required field`)
        })
    })
    job_folder_path: string | null = null;

    @attribute({
        label: t`Job Regex`,
        validation: yup.string().nullable()
    })
    job_regex: string | null = null;

    @attribute({
        label: t`Job Search Color`,
        validation: yup.string()
    })
    job_search_color: LedColorOption = 'white';

    @attribute({
        label: t`Job Search Blink Duration (minutes)`,
        validation: yup.number()
    })
    job_search_duration: number = 1;

    @attribute({
        label: t`Hot Job Color`,
        validation: yup.string()
    })
    hot_job_color: LedColorOption = 'red';

    @attribute({
        label: t`Auto Launch`,
        validation: yup.boolean()
    })
    auto_launch: boolean = false;

    @attribute({
        label: t`Maximum Launch (per minute)`,
        validation: yup.number().integer().min(1).max(20).typeError(t`Maximum Launch (per minute) is a required field`)
    })
    max_launch_records: number = 5;

    @attribute({
        label: t`API URL`,
        validation: yup.string().required(t`API URL is a required field`)
    })
    paperless_api_url: string | null = null

    @attribute({
        label: t`Order Pairing Mode`,
        validation: yup.string()
    })
    order_pairing_mode: 'manual' | 'automatic' = 'automatic'

    @attribute({
        label: t`'Find My Job' Blinking Interval`,
        validation: yup.string()
    })
    find_order_blink_speed: LedBlinkSpeedOption = 'FLASH_1_SECOND'

    @attribute({
        label: t`Hot order Blinking Interval`,
        validation: yup.string()
    })
    hot_order_blink_speed: LedBlinkSpeedOption = 'FLASH_1_SECOND'

    @computed
    get allowEditAutoLaunch(): boolean {
        return this.external_dcs_id !== null && this.job_folder_path !== null && this.paperless_api_url !== null;
    }

    @computed
    get isManualPairingMode(): boolean {
        return this.viewModel.order_pairing_mode === 'manual';
    }

    constructor() {
        super();
        super.setup();
    }
}

export const LED_COLOR_OPTIONS: LedColorOption[] = ['white', 'red', 'green', 'blue', 'yellow', 'cyan', 'magenta'];
export const BLINK_SPEED_OPTIONS: SelectOption[] = [
    {label: t`Fastest`, value: 'FLASH_MULTI'},
    {label: t`1 second`, value: 'FLASH_1_SECOND'},
    {label: t`2 seconds`, value: 'FLASH_2_SECONDS'},
    {label: t`4 seconds`, value: 'FLASH_4_SECONDS'},

]

export const SettingStore = SettingModel.store as DataStore<SettingModel>;
