import {attribute, DataStore, Model, model} from '@rx-suite/data';
import {t} from "@lingui/macro";
import {action, computed, makeObservable, runInAction} from "mobx";
import * as yup from "yup";
import {SelectOption} from "react-select-material-ui";

class TicketWorkstationStoreDataStore<T extends Model> extends DataStore<T> {

    get workstationActionOptions(): SelectOption[] {
        return [
            {label: t`Write Template Workstation`, value: 'write_template'},
            {label: t`Job Find Workstation`, value: 'order_find'}
        ]
    }

    constructor(model, transportLayer, permissionTarget, routingInfo, eager) {
        super(model, transportLayer, permissionTarget, routingInfo, eager);
        makeObservable(this);
    }
}

@model({
    transportLayer: "/api/ticket_workstations",
    permissionTarget: "TicketWorkstation",
    routingInfo: {
        create: '/settings/workstations/create',
        read: '/settings/workstations/:id/read',
        update: '/settings/workstations/:id/edit'
    },
    storeClass: TicketWorkstationStoreDataStore
})
export class TicketWorkstationModel extends Model {
    @attribute({
        label: t`Workstation Name`,
        validation: yup.string().required(t`Workstation Name is a required field.`)
    })
    name: string = "";

    @attribute({
        label: t`Associated Template`,
        validation: yup.number().when('workstation_type', {
            is: 'ticket_edit',
            then: schema => schema.typeError(t`Associated Template is a required field.`)
                .required(t`Associated Template is a required field.`),
            otherwise: schema => schema.nullable()
        })
    })
    ticket_template_id: number | null = null;

    @attribute({
        label: t`Display Order`,
        validation: yup.number().min(1).max(999).typeError(t`Display Order is a required field.`)
    })
    display_order: number = 1;

    @attribute({
        label: t`Action`,
        validation: yup.string()
    })
    action: 'write_template' | 'stage' | 'release'| 'associate' | 'mark_hot_job' | 'order_find' = 'write_template';

    @attribute()
    isLoading: boolean= false

    @action
    async callForBulkValues(values: Record<string, string>[]) {
        runInAction(() => {
            this.isLoading = true;
        })

        try {
            const response = await this.store.transportLayer.fetch(`/${this.id}/call`, {
                method: "PUT",
                headers: {"Content-Type": "application/json"},
                body: JSON.stringify({
                    values: values
                })
            });

            runInAction(() => {
                this.isLoading = false;
            })

            return await response.json();
        }catch (err){
            runInAction(() => {
                this.isLoading = false;
            })

            throw err;
        }
    }

    @computed
    get isControlWorkstation() {
        return this.action === 'stage' || this.action === 'release' || this.action === 'associate' || this.action === 'mark_hot_job';
    }

    @computed
    get isWriteTemplateWorkstation() {
        return this.viewModel.action === 'write_template'
    }

    canDelete(...args: any[]) {
        return !this.isControlWorkstation;
    }

    constructor() {
        super();
        super.setup();
    }
}

export const TicketWorkstationStore = TicketWorkstationModel.store as TicketWorkstationStoreDataStore<TicketWorkstationModel>;