module BulkAction
  class BulkActionJob < ApplicationJob
    queue_as :bulk_action

    class << self
      def perform_now(ticket_barcodes, action, source: 'SYSTEM', transaction_data: {})
        with_transactions(ticket_barcodes, action, source, transaction_data) do |transaction_ids|
          super(transaction_ids, transaction_data)
        end
      end

      def perform_later(ticket_barcodes, action, source: 'SYSTEM', transaction_data: {})
        with_transactions(ticket_barcodes, action, source, transaction_data) do |transaction_ids|
          super(transaction_ids, transaction_data)
        end
      end

      def with_transactions(ticket_barcodes_or_order_numbers, action, source='SYSTEM', transaction_data)
        inject_to_transaction_data(transaction_data)

        transaction_wrappers = build_transaction_wrappers(ticket_barcodes_or_order_numbers, action, source, transaction_data)

        unless transaction_wrappers[:successful].length == 0
          transactions = transaction_wrappers[:successful].map {|transaction_wrapper| transaction_wrapper[:transaction]}

          on_queued(transactions)

          yield(transactions.map(&:id))
        end

        transaction_wrappers
      end

      def build_transaction_wrappers(ticket_barcodes, action, source, transaction_data)
        ApiTransactionsBuilder.prepare_for_ticket_barcodes(ticket_barcodes, action, source, transaction_data)
      end

      concerning :ActionHooks do
        def inject_to_transaction_data(transaction_data)
          # Overwrite this in the child class to add data to the endpoint data before transactions are created
        end

        def on_queued(successful_transactions)
          # Overwrite this in the child class to some some actions after the transactions are created, but before the transactions are sent workers
        end
      end
    end

    def perform(transaction_ids, endpoint_data)
      @valid_transactions = {}
      @endpoint_data = endpoint_data

      load_valid_transaction_wrappers(transaction_ids)

      # Transactions could have been invalidated from releasing the tickets
      return if no_valid_transactions

      mark_transactions_ongoing(current_valid_transaction_ids)

      inject_data

      # Transactions could have been invalidated by no order data
      return if no_valid_transactions

      endpoint.new(@valid_transactions, @endpoint_data).execute
    rescue PaperlessApi::Asynchronous::Endpoints::Endpoint::EslApiError
      invalidate_transactions(current_valid_transaction_ids, "Unexpected error calling ESL API. If this issue persists please contact support.", :ESL_API_ERROR)
    rescue PaperlessApi::Asynchronous::Endpoints::Endpoint::EslApiUnreachable
      invalidate_transactions(current_valid_transaction_ids, "Cannot reach ESL API - Verify that the service is running.", :ESL_API_UNREACHABLE)
    rescue => error
      Rails::logger.error "#{self.class.name}: #{error}"
      invalidate_transactions(current_valid_transaction_ids, "Unexpected error. If this issue persists please contact support.", :UNKNOWN_ERROR)
    end

    def load_valid_transaction_wrappers(transaction_ids)
      transactions = transaction_models(transaction_ids)

      failed_ids = []

      transaction_ids.each do |transaction_id|
        transaction = transactions.detect {|transaction| transaction.id == transaction_id}

        # Rare edge case where ticket is released before a transaction is processed
        if transaction.nil?
          next failed_ids << transaction_id
        end

        @valid_transactions[transaction.id] = {
          transaction: transaction,
          data: {
            fields: []
          }
        }
      end

      if failed_ids.length > 0
        mark_transactions_failed(failed_ids, 'Transaction no longer associated to e-ticket.', :NOT_ASSOCIATED)
      end
    end

    def transaction_models(transaction_ids)
      PaperlessApiTransaction.joins(:paperless_ticket)
                             .where(id: transaction_ids)
    end

    concerning :ActionHooks do
      def inject_data
        # Overwrite this in the child class to pre-load certain data in the transaction wrappers
      end

      def on_transaction_invalidate(invalidated_transactions)
        # Overwrite this in the child class to do some when transactions are failed
      end
    end

    concerning :TransactionBroadcasting do
      def current_valid_transaction_ids
        @valid_transactions.keys
      end

      def no_valid_transactions
        @valid_transactions.length == 0
      end

      def invalidate_transactions(transaction_ids, error_message, error_code)
        return unless transaction_ids.length > 0

        invalidated = @valid_transactions.select {|transaction_id, _| transaction_ids.include?(transaction_id)}
                                         .map {|_, transaction_wrapper| transaction_wrapper[:transaction]}

        on_transaction_invalidate(invalidated)

        @valid_transactions.except!(*transaction_ids)
        mark_transactions_failed(transaction_ids, error_message, error_code)
      end

      def mark_transactions_ongoing(transaction_ids)
        transactions = PaperlessApiTransaction.where(id: transaction_ids)
        transactions.update_all(state: :ongoing)
        PaperlessApiTransaction.broadcast_multiple_later(transactions)
      end

      def mark_transactions_failed(transaction_ids, error_message, error_code)
        transactions = PaperlessApiTransaction.where(id: transaction_ids)
        transactions.update_all(
          state: :failed,
          response_body: {error: error_message, code: error_code}.to_json.to_s,
          finished_at: Time.zone.now,
          duration: 0 # transactions were never sent to ESL API successfully
        )
        PaperlessApiTransaction.broadcast_multiple_later(transactions)
      end
    end
  end
end

