module BulkAction
  module OrderBulkAction
    class LaunchOrderJob < OrderBulkActionJob
      class ErrorWithOrderDataSource < StandardError; end

      class << self
        def inject_to_transaction_data(transaction_data)
          transaction_data[:template] = TicketTemplate.launch_template
        end

        def on_queued(successful_transactions)
          order_records = order_records_for_broadcast_from_transactions(successful_transactions)
          order_records.update_all(state: :launching)
          OrderRecord.broadcast_multiple_now(order_records)
        end
      end

      def endpoint
        PaperlessApi::Asynchronous::Endpoints::Write
      end

      def inject_data
        load_initial_order_data

        inject_data_for_template_write
      end

      def on_transaction_invalidate(invalidated_transactions)
        order_records = OrderBulkActionJob.order_records_for_broadcast_from_transactions(invalidated_transactions)
        order_records.update_all(state: :failed_launch)
        OrderRecord.broadcast_multiple_later(order_records)
      end

      concerning :InitialOrderData do
        def load_initial_order_data
          begin
            order_data = load_order_lms_data

            # expected rx-link response: [{number: string, jobExists: bool, tags: {left: [], right: []}}]
            not_found = order_data.select {|job_data| job_data[:jobExists] == false}
            valid_order_data = order_data.select {|job_data| job_data[:jobExists] == true}

            mark_failed_not_found(not_found.map {|order| order[:number]})

            orders_tag_data = match_tag_data_to_transactions(valid_order_data)

            # expected rx-data response: [{number: string, order_id: number}]
            order_number_data_ids = send_orders_data_to_rx_data(orders_tag_data)

            add_order_id_to_order_tag_data(orders_tag_data, order_number_data_ids)

            save_order_record_data(orders_tag_data)

            cache_orders_tag_data(orders_tag_data)
          rescue ErrorWithOrderDataSource
            invalidate_transactions(current_valid_transaction_ids,
                                    "Failed to load order data. Verify all services running correctly",
                                    :JOB_DATA_SOURCE_ERROR)
          end
        end

        def match_tag_data_to_transactions(valid_order_data)
          order_temp_data = {}
          @valid_transactions.each do |_, order_transaction|
            order_record = order_transaction[:transaction].paperless_ticket.order_record
            transaction_order_data = valid_order_data.detect { |order_data| order_data[:number] == order_record.number }
            real_order_number = real_order_number_from_tags(transaction_order_data[:tags])

            order_temp_data[order_record.id] = {order_number: real_order_number, tags: transaction_order_data[:tags], rx_data_order_id: nil}
          end

          order_temp_data
        end

        def add_order_id_to_order_tag_data(orders_tag_data, order_number_data_ids)
          orders_tag_data.each do |order_record_id, transaction_tag_data|
            data = order_number_data_ids.detect {|order_data_id| order_data_id[:number] === transaction_tag_data[:order_number]}
            orders_tag_data[order_record_id][:rx_data_order_id] = data[:order_id]
          end
        end

        def save_order_record_data(orders_tag_data)
          record_updates = []

          orders_tag_data.each do |order_record_id, transaction_tag_data|
            # Because bulk update will not touch our in memory model, manually assign the attribute to skip reloading them entirely
            transaction_id, _ = @valid_transactions.detect {|_, order_transaction| order_transaction[:transaction].paperless_ticket.order_record.id == order_record_id}
            @valid_transactions[transaction_id][:transaction].paperless_ticket
                                                             .order_record
                                                             .assign_attributes(
                                                               {
                                                                 number: transaction_tag_data[:order_number],
                                                                 rx_data_order_id: transaction_tag_data[:rx_data_order_id]
                                                               }
                                                             )

            record_updates << {
              id: order_record_id,
              number: transaction_tag_data[:order_number], # Update the order record with the LMS provided order number
              rx_data_order_id: transaction_tag_data[:rx_data_order_id],
            }
          end
          OrderRecord.bulk_update(record_updates)
        end

        def cache_orders_tag_data(orders_tag_data)
          data = orders_tag_data.map do |order_record_id, order_data|
            {
              order_record_id: order_record_id,
              data: {
                tags: order_data[:tags]
              }.as_json # cache save expects a json object for the cache data
            }
          end
          OrderDataCache.save_order_record_id_tags(data)
        end
      end

      concerning :LmsDataLoader do
        def load_order_lms_data
          raise StandardError('No LMS selected for tag data.') if Setting.instance.external_dcs_id.nil?

          data = fetch_jobs_from_rx_link(@valid_transactions.map {|_, transaction_record| transaction_record[:transaction].paperless_ticket.order_record.launch_number})

          unless data[:success] # Connection error with LMS itself
            Rails::logger.error "Error with LMS connection: #{data[:error]}"
            raise ErrorWithOrderDataSource
          end

          data[:jobs]
        rescue => error
          Rails::logger.error "Connection to Rx-Link Failed: #{error}"
          raise ErrorWithOrderDataSource
        end

        private

        def fetch_jobs_from_rx_link(job_numbers)
          RxModule::Communicator.call_module_method_as_module(
            "rx_link",
            "/api/dcs_configs/#{Setting.instance.external_dcs_id}/multi_job_call",
            :post,
            {
              jobs: job_numbers,
              tags: Tag.pluck(:tag)
            }
          )
        end

        def real_order_number_from_tags(tags)
          # Get tag from either side
          tag = Setting.instance.order_number_tag.to_sym
          return tags[:left][tag] if tags[:left] && tags[:left][tag]
          return tags[:right][tag] if tags[:right] && tags[:right][tag]
          nil
        end

        def mark_failed_not_found(order_launch_numbers)
          failed = []
          order_launch_numbers.each do |launch_number|
            failed_transaction_id, _ = @valid_transactions.detect { |_, transaction_record|
              transaction_record[:transaction]
                .paperless_ticket
                .order_record
                .launch_number == launch_number
            }
            failed << failed_transaction_id
          end
          invalidate_transactions(failed, "Order not found (order canceled?)", :ORDER_NOT_FOUND)
        end
      end

      concerning :PostToRxData do
        def send_orders_data_to_rx_data(orders_tag_data)
          begin
            payload = {
              orders: orders_tag_data.map do |_, order_data|
                build_payload_for_order_data(order_data)
              end
            }

            data = post_orders_data_to_rx_data(payload)

            data[:orders]
          rescue => error
            Rails::logger.error "Error saving orders with Rx-Data: #{error}"
            raise ErrorWithOrderDataSource
          end
        end

        private

        def post_orders_data_to_rx_data(payload)
          RxModule::Communicator.call_module_method_internal('rx_data', "/api/orders/create_multiple", :post, payload)
        end

        def build_payload_for_order_data(order_data)
          order_data[:tags] = order_data[:tags].symbolize_keys #symbolize the top left/right level

          {
            number: order_data[:order_number],
            description: 'Created from Rx-Paperless launch.',
            order_data: payload_for_order_data(order_data[:tags]),
            processes: payload_for_processes(order_data[:tags])
          }
        end

        def payload_for_order_data(order_tags)
          data = []

          order_tags.each do |side, tags|
            next if tags.nil?
            # Unlike processes, if a tag data is shared by both eyes, still send up separately (cannot use do 'B')
            tags.each do |tag, val|
              data << {name: tag, lensside: side == :left ? "L": "R", value: val}
            end
          end
          data
        end

        def payload_for_processes(order_tags)
          [
            {
              :name => 'paperless-launch',
              :eye => side_from_tags(order_tags)
            }
          ]
        end

        def side_from_tags(tags)
          raise MissingSideForJob if tags[:left].nil? && tags[:right].nil?
          return 'R' if tags[:left].nil?
          return 'L' if tags[:right].nil?
          'B'
        end
      end
    end
  end
end
