module BulkAction
  module OrderBulkAction
    class OrderBulkActionJob < BulkActionJob
      class << self
        def perform_now(order_numbers, action, source: 'SYSTEM', transaction_data: {})
          super(order_numbers, action, source: source, transaction_data: transaction_data)
        end

        def perform_later(order_numbers, action, source: 'SYSTEM', transaction_data: {})
          super(order_numbers, action, source: source, transaction_data: transaction_data)
        end

        def build_transaction_wrappers(order_numbers, action, source, transaction_data)
          ApiTransactionsBuilder.prepare_for_order_numbers(order_numbers, action, source, transaction_data)
        end

        concerning :Broadcasting do
          def order_records_for_broadcast_from_transactions(transactions)
            # Because some transaction can failed due to no longer being associated to a ticket, remove any transaction with no paperless ticket or order record
            order_record_ids = transactions.map {|transaction| transaction.paperless_ticket&.order_record&.id}.compact
            OrderRecord.where(id: order_record_ids)
                       .left_outer_joins(:paperless_ticket)
          end

          def paperless_tickets_for_broadcast_from_transactions(transactions)
            # Because some transaction can failed due to no longer being associated to a ticket, remove any transaction with no paperless ticket
            ticket_ids = transactions.map {|transaction| transaction.paperless_ticket&.id}.compact
            PaperlessTicket.where(id: ticket_ids)
          end
        end
      end

      def transaction_models(transaction_ids)
        PaperlessApiTransaction.joins(paperless_ticket: [:order_record])
                               .where(id: transaction_ids)
      end

      concerning :TransactionsDataBuilder do
        def inject_data_for_template_write
          # Normally, we can't get to this point without an rx-data order, but the following code depends on the rx_data_order_id
          ensure_transaction_order_has_rx_data_id

          # Transactions could all be invalidated from error
          return if no_valid_transactions

          inject_template_data

          # Transactions could all be invalidated from no order data
          return if no_valid_transactions

          inject_hot_job_data
        end

        def ensure_transaction_order_has_rx_data_id
          invalid_transactions = @valid_transactions.select do |_, transaction_wrapper|
            transaction_wrapper[:transaction].paperless_ticket
                                           .order_record
                                           .rx_data_order_id
                                           .nil?
          end
          if invalid_transactions.length > 0
            invalidate_transactions(invalid_transactions.keys, "External order id is invalid. If this issue persists please contact support.", :DATA_ERROR)
          end
        end

        def inject_template_data
          raise StandardError.new('Template is not defined for inject template!') if @endpoint_data[:template].nil?

          order_records = @valid_transactions.map{|_, transaction_wrapper| transaction_wrapper[:transaction].paperless_ticket.order_record}

          begin
            order_record_data = OrderDataCache.fetch_for_order_records(order_records)
          rescue OrderDataCache::DataSourceOffline
            return invalidate_transactions(current_valid_transaction_ids,"Failed to load order data from cache. Verify that the service is running.", :RX_DATA_UNREACHABLE)
          end

          transactions_with_no_data = order_record_data
                                        .reject {|_, data| !data[:tags].nil? }
                                        .map do |order_record_id, data|
            transaction_id, _ = @valid_transactions.detect {|_, transaction_wrapper| transaction_wrapper[:transaction].paperless_ticket.order_record.id == order_record_id}
            transaction_id
          end
          invalidate_transactions(transactions_with_no_data, 'Order Data not found in cache.', :RX_DATA_ORDER_NOT_FOUND)

          order_static_data = PaperlessTicket.ticket_static_data_by_order_record(@valid_transactions.map {|_, transaction_wrapper| transaction_wrapper[:transaction].paperless_ticket.id})

          order_template_data = merge_order_data_hashes(order_record_data, order_static_data)

          order_template_payloads = @endpoint_data[:template].resolve_template_for_data_records(order_template_data)

          match_template_data_to_valid_transactions(order_template_payloads)
        end

        def inject_hot_job_data
          @valid_transactions.each do |transaction_id, transaction_wrapper|
            @valid_transactions[transaction_id][:data][:fields].concat([
                                                                         {
                                                                           zoneId: :hotJob,
                                                                           value: transaction_wrapper[:transaction].paperless_ticket.order_record.is_hot_job
                                                                         }
                                                                       ])
          end
        end

        def match_template_data_to_valid_transactions(template_data)
          @valid_transactions.each do |transaction_id, transaction_wrapper|
            order_record_id = transaction_wrapper[:transaction].paperless_ticket.order_record.id
            order_template_data = template_data[order_record_id]

            @valid_transactions[transaction_id][:data][:fields].concat(order_template_data) unless order_template_data.nil?
          end
        end

        def merge_order_data_hashes(data1, data2)
          # order data hashes are index via order_record id => :hash, so merge the inner hashes on the order_record_id
          data1.merge(data2) {|_, a,b| a.merge(b)}
        end
      end
    end
  end
end
