class ApiTransactionsBuilder
  class << self
    # For transactions that are order dependant - validates orders exists and ticket associations
    def prepare_for_order_numbers(order_numbers, transaction_type, source, transaction_data)
      ensure_transaction_type(transaction_type)

      failed = []

      valid_orders, missing_orders = validate_order_numbers(order_numbers)
      failed.concat build_failed_wrapper(
                      missing_orders,
                      :order_number,
                      "no order with matching number.",
                      :ORDER_NOT_FOUND
                      )


      paired_orders, unpaired_orders = validate_orders_associated(valid_orders)
      failed.concat build_failed_wrapper(
                      unpaired_orders,
                      :order_number,
                      "order not currently attached to e-ticket.",
                      :ORDER_NOT_ASSOCIATED
                    )

      transaction_wrappers = paired_orders.map do |order_data|
        {
          order_number: order_data[:order_number],
          transaction: PaperlessApiTransaction.prepare_transaction(order_data[:ticket_id], transaction_type, source, transaction_data)
        }
      end

      {
        successful: save_transaction_wrappers!(transaction_wrappers),
        failed: failed
      }
    end

    # For transactions that are not order dependant - validates barcode exists
    def prepare_for_ticket_barcodes(ticket_barcodes, transaction_type, source, transaction_data)
      ensure_transaction_type(transaction_type)

      valid_tickets = PaperlessTicket.where(ticket_barcode: ticket_barcodes).pluck(:id, :ticket_barcode)
                                     .map { |record| {id: record[0], ticket_barcode: record[1]} }

      valid_ticket_barcodes = valid_tickets.map {|record| record[:ticket_barcode]}
      invalid_ticket_barcodes = ticket_barcodes - valid_ticket_barcodes

      transaction_wrappers = valid_tickets.map do |ticket_data|
        {
          ticket_barcode: ticket_data[:ticket_barcode],
          transaction: PaperlessApiTransaction.prepare_transaction(ticket_data[:id], transaction_type, source, transaction_data)
        }
      end

      {
        successful: save_transaction_wrappers!(transaction_wrappers),
        failed: build_failed_wrapper(
          invalid_ticket_barcodes,
          :ticket_barcode,
          "Invalid e-ticket barcode.",
          :INVALID_BARCODE
        )
      }
    end

    private

    def ensure_transaction_type(transaction_type)
      raise StandardError.new("Invalid Transaction Type") unless PaperlessApiTransaction.valid_transaction_types.include?(transaction_type)
    end

    def validate_order_numbers(order_numbers)
      valid_orders = OrderRecord.where(number: [order_numbers]).pluck(:number)

      missing_orders = order_numbers - valid_orders

      [valid_orders, missing_orders]
    end

    def validate_orders_associated(order_numbers)
      valid = []
      not_associated = []

      OrderRecord.left_outer_joins(:paperless_ticket)
                 .where(number: [order_numbers])
                 .pluck(:number, 'paperless_tickets.id')
                 .each do |record|
        order_num, ticket_id = record

        if ticket_id.nil?
          not_associated << order_num
        else
          valid << {order_number: order_num, ticket_id: ticket_id}
        end
      end

      [valid, not_associated]
    end

    def save_transaction_wrappers!(transaction_wrappers)
      return [] if transaction_wrappers.length == 0

      transactions = transaction_wrappers.map {|order_transaction| order_transaction[:transaction]}
      PaperlessApiTransaction.insert_all!(transactions)

      # Insert all does not give us the model post-insert, so we need to re-load them manually to have the fully resolved active record
      transaction_identifiers = transaction_wrappers.map {|order_transaction| order_transaction[:transaction][:transaction_identifier]}

      transaction_records = PaperlessApiTransaction.left_outer_joins(paperless_ticket: [:order_record])
                                                   .where(transaction_identifier: transaction_identifiers)
                                                   .to_a

      transaction_wrappers.each do |transaction_wrapper|
        transaction = transaction_records.detect do |transaction|
          transaction.transaction_identifier == transaction_wrapper[:transaction][:transaction_identifier]
        end

        transaction_wrapper[:transaction] = transaction
      end

      # Broadcast all newly created transactions as insert all will not cause broadcasts
      transaction_ids = transaction_wrappers.map { |transaction_wrapper| transaction_wrapper[:transaction][:id]}
      PaperlessApiTransaction.broadcast_multiple_now(transaction_ids)

      transaction_wrappers
    end

    def build_failed_wrapper(values, identifier, error_msg, error_code)
      values.map do |value|
        wrapper = {
          error: error_msg,
          code: error_code
        }
        wrapper[identifier] = value
        wrapper
      end
    end
  end
end
