class OrderDataCache
  class DataSourceOffline < StandardError; end

  class << self
    def fetch_for_order_records(order_records)
      records = {}
      cache_misses = []

      # Get successful cache hits
      order_records.each do |order_record|
        record = Rails.cache.read(order_record.id)

        if record.nil?
          cache_misses << order_record
        else
          records[order_record.id] = record
        end
      end

      # Fetch cache misses from rx_data and add to cache
      records.merge! fetch_cache_misses(cache_misses) if cache_misses.length > 0
      records
    end

    def save_order_record_id_tags(order_record_ids_tag_data)
      records = {}

      order_record_ids_tag_data.each do |order_record_id_tag_data|
        order_record_id = order_record_id_tag_data[:order_record_id]
        tag_data = order_record_id_tag_data[:data]

        tag_data.transform_keys!(&:to_sym) # save :tags as symbol
        tag_data[:tags].transform_keys!(&:to_sym) # save :left/right as symbols

        # Cache hits contain the order tags indexed by side (i.g: {tags: {left: {"DOENG": '0', "LIND": 1.5}, right: nil}})
        Rails.cache.write(order_record_id, tag_data, expires_in: 10.minutes)

        records[order_record_id] = tag_data
      end

      records
    end

    private

    def fetch_cache_misses(order_records)
      order_ids = order_records.pluck(:rx_data_order_id)

      order_ids_tags = load_rx_data_orders_data(order_ids)

      records_to_save = []
      records = {}

      order_records.each do |order_record|
        error = order_ids_tags[order_record.rx_data_order_id]["error"]

        # Error fetching tags for job, dont cache that record
        next records[order_record.id] = {error: error} unless error.nil?

        records_to_save << {order_record_id: order_record.id, data: order_ids_tags[order_record.rx_data_order_id]}
      end

      new_records = save_order_record_id_tags(records_to_save)

      records.merge!(new_records)
    end

    def load_rx_data_orders_data(order_ids)
      resp = fetch_rx_data_orders_data(order_ids)

      resp["orders"].transform_keys(&:to_i) # make the top level keys numbers so we can access directly
    rescue Faraday::ConnectionFailed
      Rails::logger.error "[Order Cache] Unable to retrieve order data - verify connection to Rx-Data"
      raise DataSourceOffline
    end

    def fetch_rx_data_orders_data(order_ids)
      # expected rx-data response: {orders: {"10001": {"left": {...}, "right": nil}, "10002": {"error": "not_found"}}}
      RxModule::Communicator.call_module_method('rx_data', "/api/orders/bulk_order_data", :post, {
        order_ids: order_ids
      }).as_json # base module communicator symbolizes the key, but we need then as string
    end
  end
end