module PaperlessApi
  module Asynchronous
    module Endpoints
      class Endpoint
        class EslApiUnreachable < StandardError; end
        class EslApiError < StandardError; end

        def initialize(transaction_wrappers, endpoint_data={})
          @transaction_ids = transaction_wrappers.keys
          @payload = build_transactions_payload(transaction_wrappers, endpoint_data)

          inject_endpoint_data_to_payload_root(endpoint_data)
        end

        def execute
          set_request_at
          begin
            response = send_payload_to_lnc_api(url, :put, @payload)
          rescue Faraday::ConnectionFailed
            Rails::logger.warn "[ESL API] Cannot reach API - verify service is running."
            raise EslApiUnreachable
          rescue => err
            Rails::logger.error "[ESL API] Error calling API: #{err}"
            raise EslApiError
          end

          if response.status == 500
            Rails::logger.error "[ESL API] Received error from API call: #{response.body}"
            raise EslApiError
          end

          Oj.load(response.body, symbol_keys: true).to_json
        end

        private

        def set_request_at
          PaperlessApiTransaction.where(
            PaperlessApiTransaction.arel_table[:id].in(@transaction_ids)).update_all(esl_request_at: Time.zone.now)
        end

        def endpoint
          raise NotImplementedError
        end

        def build_transaction_body(transaction_wrapper, transaction_data)
          raise NotImplementedError
        end

        def inject_endpoint_data_to_payload_root(endpoint_data)
          # overwrite this to have the endpoint add endpoint-specific data to the payload
        end

        def build_transactions_payload(transaction_wrappers, transaction_data)
          payload = {tags: []}

          data = transaction_wrappers.map do |transaction_id, transaction_wrapper|
            base = {
              transaction: transaction_wrapper[:transaction].transaction_identifier,
              tagId: transaction_wrapper[:transaction].paperless_ticket.ticket_barcode
            }

            transaction_payload = build_transaction_body(transaction_wrapper, transaction_data)
            full_payload = base.merge transaction_payload

            payload[:tags] << full_payload

            {
              id: transaction_wrapper[:transaction].id,
              request_body: full_payload.to_json.to_s
            }
          end

          PaperlessApiTransaction.bulk_update(data)

          payload
        end

        def send_payload_to_lnc_api(url, method, payload)
          Faraday.send(method, url) do |req|
            req.headers['Content-Type'] = 'application/json'
            req.body = payload.to_json
          end
        end

        def url
          api_base_url = Setting.instance.paperless_api_url
          raise StandardError.new("Paperless API URL not defined") if api_base_url.nil?
          "#{api_base_url}#{endpoint}"
        end
      end
    end
  end
end
