module PaperlessApi
  module Asynchronous
    module Responses
      class Response
        RETRY_MAX_COUNT = 3

        class MissingTransactions < StandardError; end

        class << self
          def process(data)
            raise MissingTransactions if data.nil?
            data.each do |transaction_info|
              transaction_identifier = transaction_info[:transaction]
              next if transaction_identifier.nil?

              transaction_record = PaperlessApiTransaction.models_for_jbuilder
                                                          .find_by(transaction_identifier: transaction_identifier)
              if transaction_record.nil?
                Rails::logger.warn "Received status #{transaction_info[:results]} for #{transaction_identifier}. Transaction does not exist."
                next
              end

              if transaction_record.resolved?
                Rails::logger.warn "Received status #{transaction_info[:results]} for #{transaction_identifier}. Transaction already resolved (#{transaction_record.state})."
                next
              end

              response_handler_for_transaction(transaction_record, transaction_info)
            end
          end

          def response_handler_for_transaction(transaction, response)
            handler = case transaction.transaction_type.to_sym
                      when :launch
                        PaperlessApi::Asynchronous::Responses::Launch.new(transaction, response)
                      when :set_hot
                        PaperlessApi::Asynchronous::Responses::SetHot.new(transaction, response)
                      when :clear_hot
                        PaperlessApi::Asynchronous::Responses::ClearHot.new(transaction, response)
                      when :release
                        PaperlessApi::Asynchronous::Responses::Release.new(transaction, response)
                      when :set_template
                        PaperlessApi::Asynchronous::Responses::SetTemplate.new(transaction, response)
                      when :find
                        PaperlessApi::Asynchronous::Responses::Find.new(transaction, response)
                      when :stop_find
                        PaperlessApi::Asynchronous::Responses::StopFind.new(transaction, response)
                      else
                        raise StandardError.new("Unexpected Transaction Type #{transaction.transaction_type}")
                      end

              handler.process_response
          end
        end

        def initialize(transaction_record, transaction_info)
          @transaction = transaction_record
          @response = transaction_info
        end

        def process_response
          retry_count = 0
          begin
            if success?
              handle_success
            else
              handle_failure_or_cancel
            end
          rescue => e
            retry_count += 1
            if retry_count < RETRY_MAX_COUNT
              retry
            else
              Rails::logger.error "Could not complete transaction #{@transaction.transaction_identifier}: #{e}"
              @response[:results] = "ERROR"
              @response[:message] = "Unexpected error. If this issue persists, please contact support."
              handle_failure_or_cancel
            end
          end

          complete_transaction
        end

        def handle_success
          raise NotImplementedError
        end

        def handle_failure_or_cancel
          raise NotImplementedError
        end

        def success?
          return false if @response[:results].nil?

          @response[:results].upcase == 'COMPLETE'
        end

        def complete_transaction
          @transaction.update!(build_update_object)
          @transaction.broadcast_to_api_transaction_channels('update')
        end

        def build_update_object
          state = external_status_to_state(@response[:results])
          update_obj = {
            state: state,
            response_body: @response.to_json.to_s
          }
          if transaction_finished_states.include?(state)
            update_obj[:finished_at] = Time.zone.now
            update_obj[:duration] = if @transaction.esl_request_at.nil?
                                       0
                                    else
                                      (update_obj[:finished_at] - @transaction.esl_request_at).truncate(3)
                                    end
          end
          update_obj
        end

        def external_status_to_state(status)
          status = status.upcase unless status.nil?
          return :completed if status == 'COMPLETE'
          return :ongoing if status == 'PENDING'
          return :failed if status == 'ERROR'
          return :canceled if status == 'CANCELED'
          raise StandardError.new("Unexpected transaction status #{status}")
        end

        def transaction_finished_states
          [:completed, :failed, :canceled]
        end
      end
    end
  end
end
