module PaperlessApi
  module Synchronous
    class PaperlessApiSynchronousEndpoint
      class InvalidPayloadData < StandardError; end
      class InvalidExternalApiRequest < StandardError; end
      class InvalidEndpoint < StandardError; end
      class EndpointNotCalled < StandardError; end
      class ResourceNotFound < StandardError; end
      class MissingPaperlessApiUrl < StandardError; end

      class << self
        def call(data, id = nil)
          api_endpoint = self.new(data, id)
          api_endpoint.response
        end
      end

      def initialize(data, id)
        @payload = build_payload(data)

        raise InvalidExternalApiRequest if @payload.nil?

        @endpoint = build_endpoint(id)

        raise InvalidEndpoint if @endpoint.blank?

        call_external_api
      end

      def response
        raise EndpointNotCalled if @response.nil?
        @response
      end

      def endpoint
        raise InvalidEndpoint if @endpoint.blank?
        @endpoint
      end

      private

      def build_endpoint(id)
        raise NotImplementedError
      end

      def build_payload(data)
        raise NotImplementedError
      end

      def call_external_api
        response = Faraday.send(:get, url) do |req|
          req.headers['Content-Type'] = 'application/json'
          req.body = @payload.to_json
        end
        parsed_response = begin
                            Oj.load(response.body, symbol_keys: true)
                          rescue # If json parsing fails, pass through the body
                            response.body
                          end

        raise ResourceNotFound if response.status == 404
        raise parsed_response[:error] || parsed_response[:err] || parsed_response.to_s || "Unknown error - consult logs" if response.status == 500

        @response = { success: true, response: parsed_response, error: nil }
      rescue => err
        @response = { success: false, response: nil, error: err }
      end

      def url
        api_base_url = Setting.instance.paperless_api_url
        raise MissingPaperlessApiUrl if api_base_url.nil?
        "#{api_base_url}#{endpoint}"
      end
    end
  end
end
