class OrderFindQuery < ApplicationRecord
  enum blink_pattern: [:FLASH_1_SECOND, :FLASH_2_SECONDS, :FLASH_4_SECONDS, :FLASH_MULTI]
  enum color: [:white, :red, :green, :blue, :yellow, :cyan, :magenta]
  enum status: [:ready, :searching, :running, :stopping]

  belongs_to :ticket_workstation, required: true
  has_many :order_find_query_paperless_api_transactions
  has_many :paperless_api_transactions, through: :order_find_query_paperless_api_transactions

  def results
    return nil unless running?

    paperless_api_transactions.count
  end

  def execute
    begin
      update(status: :searching)

      # 1- get all order numbers that fit the ticket template requested.
      res = order_records_from_query

      # 2- Call Fetch from Rx-Data the tags from the orders
      begin
        orders_with_data = OrderDataCache.fetch_for_order_records(res)
      rescue OrderDataCache::DataSourceOffline
        return ensure_error_payload('Data Source is Offline', :DATA_OFFLINE)
      end

      # 3- verify JsonLogic against each order's tags
      order_ids = order_record_ids_that_qualify(orders_with_data)

      return ensure_error_payload('no orders matched condition', :NO_MATCHES) if order_ids.empty?

      ticket_barcodes = order_ids_into_ticket_barcodes(order_ids)

      # 4- start blinking tickets attached to orders that qualify
      transaction_wrappers = BulkAction::ToggleTicketLedJob.perform_later(ticket_barcodes,
                                                                          :find,
                                                                          transaction_data: {
                                                                            blink: true,
                                                                            blink_params: {
                                                                              color: color,
                                                                              pattern: blink_pattern,
                                                                              duration: blink_duration
                                                                            }
                                                                          })

      update(status: :running, complete_at: blink_duration.nil? ? nil : Time.zone.now.advance(minutes: blink_duration))

      paperless_api_transactions << PaperlessApiTransaction.where(PaperlessApiTransaction.arel_table[:id].in(
        transaction_wrappers[:successful].map {|d| d[:transaction].id}))

      success_response
    rescue => e
      ensure_error_payload(e.message, :UNCAUGHT_EXCEPTION)
    end
  end

  def stop
    begin
      update(status: :stopping)

      paperless_ticket_barcodes = PaperlessTicket.joins(:paperless_api_transactions)
                                         .where(PaperlessApiTransaction.arel_table[:id].in(paperless_api_transaction_ids))
                                         .pluck(:ticket_barcode)

      BulkAction::ToggleTicketLedJob.perform_later(paperless_ticket_barcodes, :find, transaction_data: {blink: false})

      cleanup

      success_response
    rescue => e
      ensure_error_payload(e.message, :UNCAUGHT_EXCEPTION, :running)
    end
  end

  def cleanup
    paperless_api_transactions.delete_all

    update(status: :ready, complete_at: nil)
  end

  def order_record_ids_that_qualify(orders_with_data)
    order_ids_that_qualify = []

    orders_with_data.each do |order_id, data|
      next if data[:error]
      order_ids_that_qualify << order_id if JsonLogic.order_matches_query(order_id, data, JSON.parse(query))
    end
    order_ids_that_qualify
  end

  def order_records_from_query
    OrderRecord.joins(paperless_ticket: :ticket_template)
               .select(
                 OrderRecord.arel_table[:rx_data_order_id],
                 OrderRecord.arel_table[:id],
                 TicketTemplate.arel_table[:template_name],
                 PaperlessTicket.arel_table[:ticket_barcode])
               .where(
      JsonLogic.build_arel_conditions_from_logic(JSON.parse(self.query)['logic'], TicketTemplate, :template_name, 'template')
    ).limit(1000).to_a #the limit here is a temp fix as we have to load all to array
  end

  def order_ids_into_ticket_barcodes(order_ids)
    OrderRecord.joins(:paperless_ticket)
               .where(OrderRecord.arel_table[:id].in(order_ids))
               .select(PaperlessTicket.arel_table[:ticket_barcode])
               .pluck(:ticket_barcode)
  end

  def ensure_error_payload(error, error_code, state_to_return_to = :ready)
    update(status: state_to_return_to)
    {error: error, code: error_code}
  end

  def success_response
    {}
  end
end