class OrderRecord < ApplicationRecord
  include PdfFileManager
  include RxModule::BulkUpdate

  before_destroy -> { purge_from_disk }

  class MissingSideForJob < StandardError; end

  has_one_attached :data_file
  has_one :paperless_ticket

  scope :active, -> { where(state: 2..) }
  scope :waiting_for_launch, -> { where(state: :waiting_for_launch) }

  enum state: [:waiting_for_launch, :failed_launch, :launching, :launched]

  scope :releasing, -> { joins(:paperless_ticket).where(paperless_ticket: {releasing: true}) }

  class << self
    def create_order_records_for_launch_numbers(unique_launch_numbers)
      raise StandardError('Duplicate number in create order records!') if unique_launch_numbers.uniq.length != unique_launch_numbers.length

      now_time = Time.zone.now

      already_created = OrderRecord.where(launch_number: unique_launch_numbers, state: [0, 2, 3]).pluck(:launch_number)
      previous_failed_records = OrderRecord.where(launch_number: unique_launch_numbers, state: [1]).pluck(:launch_number)

      numbers_to_launch = unique_launch_numbers - already_created - previous_failed_records

      order_records_to_insert = []
      numbers_to_launch.each do |launch_number|
        order_records_to_insert << {
                                      number: launch_number,
                                      launch_number: launch_number,
                                      entry_time: now_time,
                                      state: 0, # waiting_for_launch
                                      last_action_time: now_time,
                                      failed_reason: nil,
                                      rx_data_order_id: nil,
                                      is_hot_job: false,
                                      created_at: Time.zone.now,
                                      updated_at: Time.zone.now
                                   }
      end
      OrderRecord.insert_all!(order_records_to_insert) if order_records_to_insert.length > 0
      successful = OrderRecord.where(launch_number: numbers_to_launch + previous_failed_records).to_a

      failed = []
      already_created.each do |launch_number|
        failed << {order_number: launch_number, error: 'Order number is already in use.', code: :ORDER_ALREADY_LAUNCHED}
      end

      # previously failed records can be re-launched without prior deleting
      {successful: successful, failed: failed}
    end

    def attach_pdfs_to_orders(order_file_arr)
      order_file_arr.each do |order_file|
        order_file[:order].attach_pdf(order_file[:file_name])
      end
    end

    def broadcast_multiple_now(order_records_with_ticket)
      # Paperless ticket should already be joined before broadcasting to eliminate n+1 on view
      order_records_with_ticket.each do |order_record|
        ActionCable.server.broadcast "everything", {
          model: JSON.parse(order_record.render_jbuilder_partial),
          class: order_record.class.name,
          action: :update
        }
      end
    end

    def broadcast_multiple_later(order_records)
      RxModule::WorkerBroadcast.broadcast_many_later(
        models: order_records,
        action: :update,
        locator_options: {includes: :paperless_ticket}
      )
    end
  end

  def can_be_launched?
    waiting_for_launch? || failed_launch?
  end

  def can_delete?
    waiting_for_launch? || failed_launch?
  end

  def attach_pdf(file_name)
    # Active storage seems to throw when the file is being attached while in write mode, so set it in exclusive read
    # To note: using a block with File.open will close the file upon completion
    # Remove pdf file after the association
    File.open(file_path_for_file_name(file_name), 'r') do |file|
      data_file.attach(io: file, filename: File.basename(file.path))
    end
    delete_file(file_name)
  end

  def get_order_number_from_tags(tags)
    # Get tag from either side
    tag = Setting.instance.order_number_tag.to_sym
    return tags[:left][tag] if tags[:left] && tags[:left][tag]
    return tags[:right][tag] if tags[:right] && tags[:right][tag]
    nil
  end

  def download_link
    if data_file.attached?
      "/api/order_records/#{id}/download"
    end
  end

  def content_type
    "application/pdf"
  end

  def pdf_file_name
    "#{number}.pdf"
  end

  def attached_to_ticket?
    return false if paperless_ticket.nil?
    true
  end

  def to_json(*args)
    render_jbuilder_partial
  end

  def deassociate_from_ticket_and_destroy
    # NOTE: this does not invoke the release template and should only be used if the e-ticket does not need to be updated
    if paperless_ticket
      paperless_ticket.update!(order_record_id: nil, ticket_template_id: nil, releasing: false)
    end
    destroy!
  end

  private

  def purge_from_disk
    if data_file.attached?
      # For some reason Active Storage is not actually purging the file, so we'll do it manually...
      data_file.purge
    end
  end
end
