class PaperlessApiTransaction < ApplicationRecord
  include RxModule::BulkUpdate

  belongs_to :paperless_ticket
  belongs_to :ticket_template, optional: true

  has_many :order_find_query_paperless_api_transactions
  has_many :order_find_queries, through: :order_find_query_paperless_api_transactions

  enum state: [:queued, :ongoing, :completed, :failed, :canceled], _suffix: true

  enum transaction_type: [:launch, :set_hot, :clear_hot, :release, :set_template, :find, :stop_find, :unknown], _suffix: true

  scope :attached_to_ticket, -> { where.not(paperless_ticket_id: nil) }

  class << self
    def valid_transaction_types
      PaperlessApiTransaction.transaction_types
                             .keys
                             .map {|key| key.to_sym}
                             .reject {|key| key == :unknown}
    end

    def prepare_transaction(ticket_id, transaction_type, source, transaction_data)
      {
        paperless_ticket_id: ticket_id,
        state: 0, # :created
        transaction_identifier: generate_transaction_identifier,
        transaction_type: transaction_type,
        ticket_template_id: transaction_data[:template] ? transaction_data[:template].id : nil,
        request_body: {}.to_s,
        response_body: nil,
        created_at: Time.zone.now,
        updated_at: Time.zone.now,
        source: source
      }
    end

    def broadcast_multiple_later(transactions)
      RxModule::WorkerBroadcast.broadcast_many_later(
        models: transactions,
        action: :update,
        channel_class: PaperlessApiTransactionChannel,
        locator_options: {includes: {paperless_ticket: :order_record}}
      )

      RxModule::WorkerBroadcast.broadcast_many_later(
        models: transactions,
        action: :update,
        channel_class: PaperlessTicketApiTransactionsChannel,
        locator_options: {includes: {paperless_ticket: :order_record}}
      )
    end

    def broadcast_multiple_now(transaction_ids)
      transaction_records = PaperlessApiTransaction.models_for_jbuilder
                                                   .where(id: transaction_ids)
      transaction_records.each do |transaction|
        transaction.broadcast_to_api_transaction_channels('create')
      end
    end

    def models_for_jbuilder
      join_for_jbuilder.select(cols_for_jbuilder)
    end

    def join_for_jbuilder
      PaperlessApiTransaction.left_outer_joins(paperless_ticket: [:order_record])
    end

    def cols_for_jbuilder
      [
        PaperlessApiTransaction.arel_table[Arel.star],
        PaperlessTicket.arel_table[:ticket_barcode].as("associated_ticket_barcode"),
        OrderRecord.arel_table[:number].as("associated_order_number"),
        OrderRecord.arel_table[:launch_number].as("associated_order_launch_number")
      ]
    end

    private

    def generate_transaction_identifier
      SecureRandom.uuid
    end
  end

  def broadcast_to_api_transaction_channels(action)
    PaperlessApiTransactionChannel.broadcast_to(self, render_jbuilder_partial)
    PaperlessTicketApiTransactionsChannel.broadcast(self.paperless_ticket, self, action)
  end

  def cancel
    if resolved?
      errors.add(:base, :failed, message: 'transaction already resolved')
      return false
    end

    PaperlessApi::Asynchronous::Responses::Response.response_handler_for_transaction(
      self,
      {
        results: 'CANCELED'
      }
    )

    true
  end

  def resolved?
    %w[completed failed canceled].include?(state)
  end

  def to_json(*args)
    render_jbuilder_partial
  end

  def should_broadcast?
    false
  end
end
