class PaperlessTicket < ApplicationRecord
  include RxModule::BulkUpdate

  scope :in_use, -> { where.not(order_record_id: nil) }
  scope :available, -> { where(order_record_id: nil, staged: false) }
  scope :staged, -> { where(staged: true) }
  scope :releasing, -> { where(releasing: true) }

  belongs_to :order_record, optional: true, touch: true
  belongs_to :ticket_template, optional: true
  has_many :paperless_api_transactions
  has_many :ticket_static_data

  class AlreadyAttached < StandardError; end

  class << self
    concerning :OrderTicketAssociation do
      def associate_order_records_to_available(order_records)
        # todo: FIFO here?
        available_ticket_ids = PaperlessTicket.where(staged: true, order_record_id: nil)
                                              .pluck(:id)

        failed = order_records_already_launched(order_records)
        order_records = order_records - failed.map {|record| record[:order_record]}

        # Orders in the failed state can be relaunched, just doesn't re-associate them to a new ticket
        order_records_to_relaunch_with_same_ticket = order_record_to_relaunch_with_same_ticket(order_records)
        associated = order_records_to_relaunch_with_same_ticket
        order_records = order_records - order_records_to_relaunch_with_same_ticket

        associations = []
        order_records.each do |order_record|
          break if available_ticket_ids.length == 0
          next_ticket_id = available_ticket_ids.shift

          associations << {
            id: next_ticket_id,
            order_record_id: order_record.id,
            staged: false
          }
        end

        associated.concat save_ticket_order_associations(associations) unless associations.length == 0

        return {associated: associated, not_associated: order_records - associated, failed: failed}
      end

      def associate_order_records_to_specific(order_records_ticket_barcode)
        ticket_barcodes = order_records_ticket_barcode.map {|record| record[:ticket_barcode]}
        raise StandardError('Duplicate ticket barcode - Please validate uniqueness before calling associate.') if ticket_barcodes.uniq.length != ticket_barcodes.length

        failed = order_records_already_launched(order_records_ticket_barcode.map{|record| record[:order_record]})
        failed_order_records = failed.map {|record| record[:order_record]}
        order_records_ticket_barcode = order_records_ticket_barcode.select {|record| failed_order_records.exclude?(record[:order_record])}

        tickets = PaperlessTicket.where(ticket_barcode: ticket_barcodes)
                                 .pluck(:id, :ticket_barcode, :order_record_id)
                                 .map {|data| {ticket_id: data[0], ticket_barcode: data[1], order_record_id: data[2]}}

        associations = []
        associated = []
        order_records_ticket_barcode.each do |record|
          ticket_data = tickets.detect {|data| data[:ticket_barcode] == record[:ticket_barcode]}

          if ticket_data.nil?
            failed << {order_record: record[:order_record], error: "No E-Ticket identifier matches #{record[:ticket_barcode]}", code: :INVALID_ETICKET_IDENTIFIER}
            next
          end

          # When relaunching failed work, make sure it is being assigned to the same ticket
          unless record[:order_record].paperless_ticket.nil?
            if record[:order_record].paperless_ticket.id != ticket_data[:ticket_id]
              failed << {order_record: record[:order_record], error: 'Order already assigned to a different ticket', code: :ORDER_ASSIGNED_DIFFERENT_ETICKET}
              next
            end
          end

          if ticket_data[:order_record_id] && ticket_data[:order_record_id] != record[:order_record].id
            failed << {order_record: record[:order_record], error: 'E-Ticket already in use', code: :ETICKET_IN_USE}
            next
          end

          associations << {
            id: ticket_data[:ticket_id],
            order_record_id: record[:order_record].id,
            staged: false
          }
        end

        associated.concat save_ticket_order_associations(associations) unless associations.length == 0

        return {successful: associated, failed: failed}
      end

      private

      def order_record_to_relaunch_with_same_ticket(order_records)
        prior_failed_ids = order_records.select {|order_record| order_record.state == 'failed_launch'}
                                    .map {|order_record| order_record.id}

        return [] if prior_failed_ids.length == 0

        associated = PaperlessTicket.joins(:order_record).where(order_record: {id: prior_failed_ids}).pluck('order_record.id')

        order_records.select{|order_record| associated.include?(order_record.id)}
      end

      def save_ticket_order_associations(associations)
        PaperlessTicket.bulk_update(associations)

        order_record_ids = associations.map{|association| association[:order_record_id]}
        ticket_ids = associations.map{|association| association[:id]}

        # Order is paired - it is now considered 'launching'
        OrderRecord.where(id: [order_record_ids]).update_all(state: :launching)

        # When tickets are newly associated to an order, we nil previous transaction.
        # This lets us view ticket transactions for a time after a ticket was released.
        PaperlessApiTransaction.where(paperless_ticket_id: ticket_ids).update_all(paperless_ticket_id: nil)

        OrderRecord.joins(:paperless_ticket).where(id: order_record_ids).to_a
      end

      def order_records_already_launched(order_records)
        cannot_associate = order_records.select {|order_records| order_records.state == 'launching' || order_records.state == 'launched'}

        cannot_associate.map do |order_record|
          {order_record: order_record, error: 'Order is already associated', code: :ORDER_ALREADY_ASSOCIATED}
        end
      end
    end

    def ticket_static_data_by_order_record(ticket_ids)
      static_data = TicketStaticDatum.joins(paperless_ticket: [:order_record])
                                     .where(paperless_ticket_id: ticket_ids)
                                     .pluck('order_records.id', :name, :value)

      order_static_data = {}
      static_data.each do |record|
        order_record_id, field_name, field_value = record
        order_static_data[order_record_id] = {static_data: []} if order_static_data[order_record_id].nil?
        order_static_data[order_record_id][:static_data] << {name: field_name, value: field_value}
      end

      order_static_data
    end

    def broadcast_multiple_now(paperless_tickets)
      paperless_tickets.each do |ticket|
        ActionCable.server.broadcast "everything", {
          model: JSON.parse(ticket.render_jbuilder_partial),
          class: ticket.class.name,
          action: :update
        }
      end
    end

    def broadcast_multiple_later(paperless_tickets)
      RxModule::WorkerBroadcast.broadcast_many_later(
        models: paperless_tickets,
        action: :update
      )
    end
  end

  def current_order_number
    return nil if order_record_id.nil?
    order_record.number
  end

  def state
    if attached?
      :in_use
    else
      if staged
        :staged
      else
        :available
      end
    end
  end

  def stage
    raise AlreadyAttached if attached? # cannot stage a ticket that is attached to an order
    return if staged

    paperless_api_transactions.update_all(paperless_ticket_id: nil)

    update(staged: true)
  end

  def attached?
    return false if order_record_id.nil?
    true
  end

  def latest_launch_transaction
    paperless_api_transactions.where(transaction_type: :launch).order(created_at: :desc).first
  end

  def to_json(*args)
    render_jbuilder_partial
  end
end
