class TicketTemplate < ApplicationRecord
  has_many :ticket_template_fields
  has_many :ticket_workstations
  has_many :paperless_tickets
  has_many :order_record
  belongs_to :workflow, :optional => true

  class InvalidTemplate < StandardError; end

  class << self
    def seed_defaults
      TicketTemplate.create_with(process_name: "launch", template_name: "launch", system_default: true)
                    .find_or_create_by!(process_code: TicketTemplate.launch_template_code)
      TicketTemplate.create_with(process_name: "release", template_name: "", system_default: true)
                    .find_or_create_by!(process_code: TicketTemplate.release_template_code)
    end

    def launch_template_code
      'launch'
    end

    def release_template_code
      'release'
    end

    def launch_template
      TicketTemplate.where(process_code: launch_template_code).first
    end

    def release_template
      TicketTemplate.where(process_code: release_template_code).first
    end

    def available_to_external
      TicketTemplate.all
                    .reject {|template| template.system_default}
                    .map do |template|
        {
          name: template.template_name
        }
      end
    end
  end

  concerning :ResolveTemplate do

    # Expects a hash as such {identifier => {tags?: {left?: [], right?: []}, static_data?: [{name:, value:}]}}
    # Returns a hash with the same root identifier and the template fields for that identifier {identifier => [fields]}
    def resolve_template_for_data_records(data_records)
      template_fields = resolved_template_fields

      record_fields = {}

      data_records.each do |identifier, record_data|
        record_fields[identifier] = match_fields_with_record_data(template_fields, record_data)
      end

      record_fields
    end

    private

    def resolved_template_fields
      ticket_template_fields.left_outer_joins(:tag).pluck(
        :zone_identifier,
        :field_type,
        :tag,
        :side,
        :default_value, # default value is from joins Tag - tag default
        :static_field,
        :static_default,
        :formatter_regex
      ).map do |field|
        field_type = field[1].to_sym
        {
          zoneId: field[0],
          type: field_type,
          resolver: if field_type == :job_data
                      {
                        tag: field[2],
                        side: field[3].to_sym,
                        default_value: field[4],
                      }
                    else
                      {
                        static_name: field[5],
                        default_value: field[6]
                      }
                    end,
          regex: field[7]
        }
      end
    end

    def match_fields_with_record_data(template_fields, record_data)
      fields = []

      template_fields.each do |template_field|
        field = {
          zoneId: template_field[:zoneId],
          value: if template_field[:type] == :job_data
                   resolve_job_data_field(template_field, record_data[:tags])
                 elsif template_field[:type] == :ticket_data
                   resolve_ticket_static_data_field(template_field, record_data[:static_data])
                 end
        }

        unless template_field[:regex].blank?
          field[:value] = field[:value].to_s[/#{field[:regex]}/]
        end

        fields << field
      end

      fields
    end

    def resolve_job_data_field(template_field, tag_data)
      return resolve_default_value(template_field) if tag_data.nil?


      side = if template_field[:resolver][:side] == :either
               tag_data[:left].nil? ? :right : :left
             else
               template_field[:resolver][:side]
             end

      side_data = tag_data[side]

      return resolve_default_value(template_field) if side_data.nil?

      tag_value = side_data[template_field[:resolver][:tag]]

      return resolve_default_value(template_field) if tag_value.nil?

      tag_value
    end

    def resolve_ticket_static_data_field(template_field, static_data)
      return resolve_default_value(template_field) if static_data.nil?

      static_field = static_data.detect {|data| data[:name] == template_field[:resolver][:static_name]}

      return resolve_default_value(template_field) if static_field.nil?

      static_field[:value]
    end

    def resolve_default_value(template_field)
      template_field[:resolver][:default_value] || TicketTemplateField.blank_field_value
    end
  end
end
