class TicketWorkstation < ApplicationRecord
  #include OrderDataCache
  include NormalizeTicketBarcode

  belongs_to :ticket_template, optional: true
  has_many :order_find_queries

  enum action: [:write_template, :stage, :release, :associate, :mark_hot_job, :order_find]

  class << self
    def seed_defaults
      TicketWorkstation.create_with(name: "Stage Ticket", ticket_template: nil, display_order: 0).find_or_create_by!(action: "stage")
      TicketWorkstation.create_with(name: "Release Ticket", ticket_template: nil, display_order: 1000).find_or_create_by!(action: "release")
      TicketWorkstation.create_with(name: "Associate Ticket", ticket_template: nil, display_order: 0).find_or_create_by!(action: "associate")
      TicketWorkstation.create_with(name: "Mark Hot Job", ticket_template: nil, display_order: 1001).find_or_create_by!(action: "mark_hot_job")
    end
  end

  def call(params)
    case
    when write_template?
      write_template(params)
    when stage?
      stage(params)
    when release?
      release(params)
    when associate?
      associate(params)
    when mark_hot_job?
      mark_hot_job(params)
    end
  end

  private

  def write_template(params)
    order_numbers = order_numbers_from_params(params)

    transaction_wrappers = BulkAction::OrderBulkAction::ChangeTemplateJob.perform_later(
      order_numbers,
      :set_template,
      transaction_data: {template: self.ticket_template}
    )

    response_for_transaction_wrappers(transaction_wrappers)
  end

  def release(params)
    order_numbers = order_numbers_from_params(params)

    transaction_wrapper = BulkAction::OrderBulkAction::ReleaseOrderJob.perform_later(
      order_numbers,
      :release,
    )

    response_for_transaction_wrappers(transaction_wrapper)
  end

  def mark_hot_job(params)
    order_numbers = order_numbers_from_params(params)

    transaction_wrapper = BulkAction::OrderBulkAction::ToggleHotOrderJob.perform_later(
      order_numbers,
      :set_hot,
      transaction_data: {set_hot: true}
    )

    response_for_transaction_wrappers(transaction_wrapper)
  end

  def response_for_transaction_wrappers(order_transaction_data)
    {
      transaction_ids: order_transaction_data[:successful].map do |transaction_data|
        transaction_data[:transaction].id
      end,
      failed_values: order_transaction_data[:failed].map do |transaction_data|
        {
          order_number: transaction_data[:order_number],
          error: transaction_data[:error],
          code: transaction_data[:code]
        }
      end
    }
  end

  def order_numbers_from_params(params)
    params = params.permit(values: [:order_number])
    params[:values].map do |param|
      param[:order_number]
    end
  end

  concerning :StageTicket do
    def stage(params)
      failed = []
      ids = []

      stage_params(params).map do |row|
        row[:ticket_barcode] = normalize_ticket_barcode(row[:ticket_barcode])

        ticket = PaperlessTicket.find_by(ticket_barcode: row[:ticket_barcode])
        if ticket.nil?
          failed << wrap_row_with_error(row,'Does not exist')
        elsif ticket.staged
          failed << wrap_row_with_error(row, 'Already staged')
        else
          ids << ticket.id
        end
      end

      PaperlessTicket.where(id: ids).update_all(staged:true) if ids.size > 0

      {transaction_ids: [], failed_values: failed}
    end

    private

    def wrap_row_with_error(row, error)
      {**row, error: error}
    end

    def stage_params(params)
      params.permit(values: [:ticket_barcode])['values']
    end
  end

  concerning :AssociateTicket do
    def associate(params)
      params = launch_params(params)

      results = OrderLaunch.with_order_number_and_ticket_barcode(params)

      response_for_launch_transaction_wrappers(results)
    end

    private

    def response_for_launch_transaction_wrappers(order_transaction_data)
      #Association stations use the launch number instead of the order number
      {
        transaction_ids: order_transaction_data[:successful].map do |transaction_data|
          transaction_data[:transaction].id
        end,
        failed_values: order_transaction_data[:failed].map do |transaction_data|
          {
            launch_number: transaction_data[:order_number],
            error: transaction_data[:error],
            code: transaction_data[:code]
          }
        end
      }
    end

    def launch_params(params)
      params = params.permit(values: [:ticket_barcode, :launch_number])
      params[:values].map do |param|
        {
          ticket_barcode: param[:ticket_barcode],
          number: param[:launch_number]
        }
      end
    end
  end
end
